// wrapped by build app
define("dojoe/messagedialog/MessageDialog", ["dijit","dojo","dojox","dojo/i18n!dojoe/messagedialog/nls/MessageDialog","dojo/require!dijit/_Widget,dijit/Dialog,dijit/form/CheckBox,dijit/form/Button,dojo/i18n,dojox/layout/ResizeHandle,dojoe/Util"], function(dijit,dojo,dojox){
/*********************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* Tivoli Widget Library
*
* (C) Copyright IBM Corp. 2013.  All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
************************************************************ {COPYRIGHT-END} **/
dojo.provide("dojoe.messagedialog.MessageDialog");

dojo.require("dijit._Widget");
dojo.require("dijit.Dialog"); 
dojo.require("dijit.form.CheckBox");
dojo.require("dijit.form.Button");
dojo.require("dojo.i18n");
dojo.require("dojox.layout.ResizeHandle");
dojo.require("dojoe.Util");

//Localization of resource bundles
dojo.requireLocalization("dojoe.messagedialog", "MessageDialog");


dojo.declare("dojoe.messagedialog.MessageDialog",[ dijit._Widget ],
{
	// summary:  MessageDialog is a widget to show message dialogs of type error,warning,info,confirm. 
	// description: MessageDialog widget uses the dijit.Dialog for its implementation.
	
	//theme: String
	// defined the theme for Message Dialog
	theme:null,	
	
	//resources:object
	//Reference to the resources
	resources_:null,
	 
    // resizeAxis: String
    //  x | xy | y to limit pane's sizing direction.
    resizeAxis: "xy",

	// type: String
    // defines the type of message dialog ("Error","Critical", "Warning", "Info", "Confirm").
	type: "Error",
	
	// portlet title associated w/ this message
	portletTitle: null,

	// message: String
	//the displayable translated message
	message: null,
	
	//Richa 18thJan13 #62250
	// target: DOMNode
	// The target to on top of which the message dialog would be shown
	target: null,
	
	// messageId: String
	//the displayable translated message's unique identifier
	messageId: null,
	
	// width: String
	//gives the pixel width of the dialog (Default: 500px)
	width: "380px",
	
	// height: String
	//gives the pixel height of the dialog (Default: 200px)
	height: "250px",
	
	// buttons: String Array
	//a list of buttons to display (Default: OK)
	buttons: [],
	
	// callback: Function
	//a callback function when the dialog is closed
	callback: null,
	
	// parameters to pass on callback when dialog closes
	callbackParms: null,
	
	// url: String
	//a URL that links the message to an InfoCenter page
	url: null,
	
	// windowName: String
	//window.open(URL,name,specs,replace)
	windowName: '',
	// windowSpecs: String
	windowSpecs: '',
	// windowReplace: String
	windowReplace: '',
	
	// show "Never show again checkbox"
	showCheckbox: false,
	
	// contentIsolation: boolean parameter to open PopupUtil in content isolation mode in portlets
	contentIsolation: false,
	
	// Destroy the dialog on close
	destroyOnClose: true,

	// enableResizeHandle: boolean
	//a flag to enable/disable the resize handle, default is true
	enableResizeHandle: true,
	
	// namespace: user defined namespace for widget
	namespace :"",
	
	dir : "",

  // The dialog footer that contains the buttons
  footer: null,
	
	constructor: function() {
		// summary: An extension to the Dojo Dialog widget that supports 
		//      advanced features.
		// resourceMessages_: JSON
		//		Resource bundle.
		// dialog_: dojo.widget.FloatingPane
		//		An instance of the Dialog widget.
		// title_: String
		//		Title bar string.
		// icon_: String
		//		The dialog icon's uri.
		// buttonPressedId_: String
		//		The ID of the button pressed; this value is passed to the callback function.

	    this.resources_ = dojo.i18n.getLocalization("dojoe.messagedialog", "MessageDialog",this.lang);
		
		if (this.contentIsolation == true) {
			var statusAr = startsWith(dojoConfig.locale,"ar");
			var statusHe = startsWith(dojoConfig.locale,"he");
			if (statusAr == true || statusHe == true) {
				this.dir = "rtl";
			}
		}
	    
	    this.dialog_ = null;
	    this.title_ = null;
	    this.icon_ = null;
	    this.buttonPressedId_ = -1;
	},


	postCreate: function() { 
		// summary:  Creates and initialises the message Dialog as per its type.
		// tags: public
	    if (this.buttons.length === 0) {
	        this.buttons.push(this.resources_.DIALOG_BUTTON_OK);
	    }

		var className = null;
		//Bobby:20Dec11 - changing all images to use latest in guidelines - BEGIN		
	    if (this.type.toLowerCase() == "error" ||this.type.toLowerCase() == "critical") {
	    	if (this.portletTitle) {
		    	this.title_ = dojo.string.substitute(this.resources_.MESSAGE_TITLE, { 0: this.resources_.DIALOG_ERROR, 1: this.portletTitle });
	    	} else {
		        this.title_ = this.resources_.DIALOG_ERROR;
		    }
	        this.icon_ = dojo.moduleUrl("dojoe.common", "themes/images/st34critical_shad.png");
	        className = "critical";
	    }
	    else if (this.type.toLowerCase() == "warning") {
	    	if (this.portletTitle) {
		    	this.title_ = dojo.string.substitute(this.resources_.MESSAGE_TITLE, { 0: this.resources_.DIALOG_WARNING, 1: this.portletTitle });
	    	} else {
		        this.title_ = this.resources_.DIALOG_WARNING;
		    }
	        this.icon_ = dojo.moduleUrl("dojoe.common", "themes/images/st34minorwarning_shad.png");
	        className = "warning";
	    }
	    else if (this.type.toLowerCase() == "confirm") {
	    	if (this.portletTitle) {
		    	this.title_ = dojo.string.substitute(this.resources_.MESSAGE_TITLE, { 0: this.resources_.DIALOG_CONFIRM, 1: this.portletTitle });
	    	} else {
		        this.title_ = this.resources_.DIALOG_CONFIRM;
		    }
	        //Richa-14thAug12 - Changed the Confirm icon from unknown to warning.
	        this.icon_ = dojo.moduleUrl("dojoe.common", "themes/images/st34minorwarning_shad.png");
	        className = "confirm";
	    }
	    else if (this.type.toLowerCase() == "validation") {
	    	if (this.portletTitle) {
		    	this.title_ = dojo.string.substitute(this.resources_.MESSAGE_TITLE, { 0: this.resources_.DIALOG_VALIDATION, 1: this.portletTitle });
	    	} else {
		        this.title_ = this.resources_.DIALOG_VALIDATION;
		    }
	        this.icon_ = dojo.moduleUrl("dojoe.common", "themes/images/st34success_shad.png");
	        className = "validation";
	    }
	    else { // if (this.type.toLowerCase() == "info") {
	    	if (this.portletTitle) {
		    	this.title_ = dojo.string.substitute(this.resources_.MESSAGE_TITLE, { 0: this.resources_.DIALOG_INFO, 1: this.portletTitle });
	    	} else {
		        this.title_ = this.resources_.DIALOG_INFO;
		    }
	        this.icon_ = dojo.moduleUrl("dojoe.common", "themes/images/st34informational_shad.png");
	        className = "info";
	    }
	    //Bobby:20Dec11 - changing all images to use latest in guidelines - END
		
		var div = document.createElement('div'); 
		document.body.appendChild(div);
		var theme=this.theme;
		var dir = this.dir;
        this.dialog_ = new dijit.Dialog (
			                             { title: this.title_,
											 							 dir: dir,
		                                 style: "width: " + this.width },
		                                 div); 
		/* HACK: Adjust the base class of the dialog to includ dojoeMessageDialog so we can style the dialog 
		         independent of rest of dijits. */ 
     this.dialog_.baseClass = this.dialog_.baseClass + " dojoeMessageDialog";

		var dojoAttachpoint = this.dialog_.containerNode;
		this.dialog_.closeButtonNode.setAttribute("title", this.resources_.DIALOG_BUTTON_CLOSE);
		this.dialog_.closeButtonNode.setAttribute("tabIndex", 0);
		dojo.connect(this.dialog_.closeButtonNode,'onkeypress',this,'onKeyPress');
		dojo.style(div,'class',theme);
		dojoAttachpoint.innerHTML = this.getContent(className,theme);
		// dojo.connect(dijit.Dialog.prototype, "postMixInProperties", function(){ this.buttonCancel="Close" });
    
		// Create the dialog footer and buttons
		this.footer = dojo.create( "div", { className: "dijitDialogPaneActionBar " + className, id: this.id + "_footer" } );
		
		var i, button;
		for ( i = 0; i < this.buttons.length; i++) {
			button = new dijit.form.Button( { label: this.buttons[i],
																 id: this.id + "mmd_button_" + i,
																 onClick: dojo.hitch( this, this.onClick ) } );
			//console.log( "[PostCreate] New Button ", button );
			this.footer.appendChild( button.domNode );   
		}
		// Add the buttons to the bottom of the dialog's DOM
	//	dojo.place( this.footer, this.dialog_.domNode, "last" );
		/*Richa 13thJan13 #62408 - Placing the footer at the end of the ContainerNode, else the accessibility of the dialog box is broken*/
		dojo.place( this.footer, this.dialog_.containerNode, "last" ); //Richa
		
		if(this.enableResizeHandle == true){
			//Build and place resize handle       
			var resizeHandleDOM = dojo.doc.createElement( 'span' );
			resizeHandleDOM.className = "dojoxFloatingResizeHandle";    
			dojo.place( resizeHandleDOM, this.dialog_.domNode, "last" );		
			this.resizeHandle = new dojox.layout.ResizeHandle(
			{ 
				targetId: this.dialog_.id, 
				resizeAxis: this.resizeAxis 
			}, resizeHandleDOM );  
			this.onResizeListener = dojo.connect( this.dialog_, "resize", this, "customLayout" );			
		}
	 
     this.onCloseListener  = dojo.connect( this.dialog_, "hide", this, "_onHide" );
	    
	},
	
    /**************************************************************************
    * The dialog has been hidden.  Wait 500 miliseconds to destroy the dialog.
    *
    * HACK -- Can't figure out a better way to destroy the dialog on close.
    **************************************************************************/
    _onHide:function()
    {// summary: Closes the MessageDialog .
		// tags: private
      setTimeout( dojo.hitch( this, "destroy" ), 500 );
    },  
	
	onKeyPress: function (e) {
		if (e.keyCode == 13 || e.keyCode == 32 || e.keyCode == 0) { //  Space keycode is 32 on IE8 and 0 on mozilla.  Enter keycode is 13 for both. 
			this.dialog_.hide();
		}
	},
	
	onClick: function(/* HTMLEvent */e) {
    	// summary:  Calls the callback on click of Ok else destroys.
		// tags: public
		var index = this.id.length + "mmd_button_".length;
		//55987: click on the edge of the "YES" button from cancel confirmation dialog of wizard would not take effective
		// Latha :  if we click on the edge of the button the target id is coming as empty. 
		var targetId=e.target.id;
		if(targetId == ""){
			if(e.target.firstChild){
				targetId=e.target.firstChild.id;
			}
		}
		this.buttonPressedId_ = parseInt(targetId.substring(index), 0);
	    //console.log ("MessageDialog.onClick: buttonPressedId_: " + this.buttonPressedId_);
	    var checkbox = dojo.byId (this.id + "neverShowAgain");
	    var checked = checkbox ? checkbox.checked : false;

	    this.dialog_.hide();
	    
	    if (this.callback) {
	    	this.callback(this.buttonPressedId_, this.messageId, checked, this.callbackParms);
	    }
    },

	show: function() {
    	// summary:  Displays the messageDialog.
		// tags: public
	    if( this.contentIsolation == true ){
			// Get the namespace identifying the content that launched the context menu
			//console.info("Inside Show if PopupUtil Undefined");
			var namespace="";
			if(this.namespace != null)
			 namespace = this.namespace;
			
			//Close (just in case) and then open the popup covering the whole page
			//console.info("Opening Popup namespace "+namespace);
			var xcord=100;
			var ycord=100;
			var dialogNamespace=namespace+'dojoe.messagedialog.MessageDialog';
			var old_popup_iframe = PopupUtil.ClosePopup( namespace+'dojoe.messagedialog.MessageDialog' );
			//Changes for fixing Content Isolation issue with Claro theme
			var theme = dojo.body().className;
			//var theme=this.theme; 
			
			//Changes to fix CSBUG #158121
			/*var popupUrl =dojo.baseUrl+'../html/MessageDialogPopUp.html?PopupID='+dialogNamespace+'&eLeft='+xcord+'&eTop='+ycord+'&locale='+dojoConfig.locale+"&theme="+theme;*/
			
			var popupUrl = require.toUrl('dojoe/messagedialog/MessageDialogPopUp.html?PopupID='+dialogNamespace+'&eLeft='+xcord+'&eTop='+ycord+'&locale='+dojo.locale+"&theme="+theme);
			/*var popupUrl =this.baseUrl+'sample/jsp/html/MessageDialogPopUp.html?PopupID='+dialogNamespace+'&eLeft='+xcord+'&eTop='+ycord+'&locale='+dojoConfig.locale+"&theme="+theme;*/
			//console.info("url "+popupUrl);
			var popup_iframe = PopupUtil.OpenPopup( 
				namespace, 
				namespace+'dojoe.messagedialog.MessageDialog', 
				true, 
				null, 
				null, 
				null, 
				null, 
				popupUrl, 
				null
			);
			//console.info("after popup opening code");
			//Initialize the contextMenu when the iframe popup is finished loading
			var tThis = this;
			if( popup_iframe.addEventListener ){
				popup_iframe.addEventListener(
					"load", 
					function(){
						popup_iframe.contentWindow.makeMessageDialog( tThis );
					}, 
					false
				);
			}
			else if (popup_iframe.attachEvent){
				popup_iframe.attachEvent(
					"onload", 
					function(){ 
						popup_iframe.contentWindow.makeMessageDialog( tThis);
					}
				);
			}
		}
		else{
			this.dialog_.show();
			this.customLayout();
			
			/*Richa,18thJan13 #62250
			//Based on the target node passed, the x and y coordinates are calculated
			//and the message Dialog is positioned according to that.*/
			
			var domNode = this.target;
			//Only if the target node is passed/notnull, all the calculations for the dialog positioning
			//will be done
			if (domNode){
				var co = dojo.coords(domNode);
				//size of target node
				var margin = dojo.marginBox(domNode);
				// size of the dialog box
				var dialogSize = dojo.contentBox( this.dialog_.domNode );
				
				//calculate the x and y coordinates
				if (dialogSize.w > margin.w )
					var xco = co.x-(dialogSize.w-margin.w)/2;
				else
					var xco = co.x+(margin.w-dialogSize.w)/2;
					
				if (dialogSize.h > margin.h )
					var yco = co.y-(dialogSize.h-margin.h)/2;
				else
					var yco = co.y+(margin.h-dialogSize.h)/2;
				
				//If the x and y coordinates are negative, make them 0 so that the 
				//dialog box does not move out of the window area	
				if(xco < 0)
					xco=0;
				
				if(yco < 0)
					yco=0;
					
				dojo.style(this.dialog_.domNode,'top',yco+'px');
				dojo.style(this.dialog_.domNode,'left', xco+'px');
			}
			//Richa 18thJan13 #62250 END
			this.correctPNG();
        }
	},

      customLayout: function() 
      {
		/* summary: Resizes the Message Dialog.  We only adjust the height of the dialog and allow the containing table 
		   to keep the width at 100%. This appears to work for IE 8, FireFox, and Chrome. */
		// tags: public
		
		var messageDiv = dojo.byId( this.id + '_messagediv' );
		var dialogSize = dojo.contentBox( this.dialog_.domNode );

		var maxHeight = 0.5 * this._getBodyHeight();

    // IE 8 was causing some bogus numbers to appear when the dialog first open, if that happens ignore the caller.
    if( dialogSize.w < 0 || dialogSize.h < 0 ){
			console.log( "[customLayout] Ignoring resize request since dialogSize is negative. " );
			return;
		}

	if( dialogSize.h > maxHeight){
		//console.log( "[customLayout] Setting the maximum height: "+maxHeight );
		dojo.style( this.dialog_.domNode, "height", maxHeight + "px");
		dialogSize.h = maxHeight;
	}
		
	/*Richa 9thJan12 #62170 : Setting the minimum Dialog height so that the Ok button is not hidden*/
	minDialogSize = 230;
	if( dialogSize.h < minDialogSize){
		dojo.style( this.dialog_.domNode, "height", minDialogSize + "px");
		dialogSize.h = minDialogSize;
	}
	
		//44160: TED - dojoe.messagedialog.MessageDialog: Wrong Size Calculated
		//Bobby Joseph: 27Jun2012, using marginBox instead, since OneUI has margins in title bar
		var titleBarSize = dojo.marginBox( this.dialog_.titleBar );
		
		// Start collect information about the various elements of the dialog box
		var cBox = dojo.marginBox( this.dialog_.containerNode );
    var msgBox = dojo.marginBox( messageDiv );
		var fBox = dojo.marginBox( this.footer );
		var newSize = { h:0 };
		
		/* Debug information to validate calculations or if funny things happen 
    console.log( "[customLayout] =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=" );
    console.log( "[customLayout] dialogSize [ w=" + dialogSize.w + ", h="+ dialogSize.h + "]" );
    console.log( "[customLayout] fBox [ w=" + fBox.w + ", h=" + fBox.h + "]" );
		console.log( "[customLayout] cBox [ w=" + cBox.w + ", h=" + cBox.h + "]" );
		console.log( "[customLayout] msgBox [ w=" + msgBox.w + ", h=" + msgBox.h + "]" );
	  console.log( "[customLayout] titleBarSize [ w=" + titleBarSize.w + ", h=" + titleBarSize.h + "]" ); */
		
		/* We subtract the current message box size from the content box size to figure out how much margins, padding, and
		   surrounding content are taking up space so we can correctly adjust the width and height of the message description
			 based on the detla we're given by the resize event. */
	/*Richa 13thJan13 #62408 - Removing footer box height as it is a part of container node now */
	//	newSize.h = dialogSize.h - titleBarSize.h - fBox.h - (cBox.h - msgBox.h);
	newSize.h = dialogSize.h - titleBarSize.h - (cBox.h - msgBox.h);
	//Richa : 30thNov12 ,#58811 - Increasing the height of messageDiv by 1px else it was trigerring the resize event again and again
	// till the minHeight was reached. Happening only in IE-oneui theme
	
		var theme = dojo.body().className;
		var index = theme.indexOf( "oneui" );
		if (index != -1){
			newSize.h = newSize.h+1;
		}
		// Use the height of the message-id to be the minimum height of the message area.  That should give
		// us at least one line of text before scrolling. 
		// Fall back on the dialog title as the min height.  Might be bigger than we need but it is at least something.
		var msgId = dojo.query( ".message-id", this.dialog_.domNode )[0] || 
		            dojo.query( ".dijitDialogTitle", this.dialog_.domNode )[0] || null;
		
		
		if( msgId !== null ){	
		  var minHeight = dojo.marginBox( msgId );

      // Make sure we use a good min size since the user could resize the box too small to fit the content. 
		  newSize.h = Math.max( minHeight.h, newSize.h );	
		} else {
			console.log( "[customLayout] Unable to determine the min height of a text string all plan exhasted.", msgId );
		}
	
	  //console.log( "[customLayout] final size [ h=" + newSize.h + "]" );
		
    dojo.contentBox( messageDiv, newSize );
	},

	_getBodyHeight: function() {
			return ((dojo.window.getBox().h != undefined) ? dojo.window.getBox().h : window.innerHeight);
		},
	
	getContent: function(className,theme) {
		// summary:  gets the content to show inside messageDialog.
		// tags: public
		var innerHTML = "";
		
		innerHTML += "<div class='messagedialog'>";
		if (this.showCheckbox === true) {
			innerHTML += "<table class=\"" + className + "checkbox\" role=\"presentation\">";
		}else innerHTML += "<table class=\"" + className + "\" role=\"presentation\">";
		innerHTML += "<tr valign=\"top\">";
		innerHTML += "<td>";
		innerHTML += "<img role='img'  align=\"top\" src=\"" + this.icon_ + "\" tabIndex=0 alt=\"" + this.title_ + "\" title=\"" + this.title_ + "\" width='34' height='34' >";
		innerHTML += "</img>"		
		innerHTML += "</td>";
		innerHTML += "<td>";		

		innerHTML += "<table style='width: 100%;' role=\"presentation\">";
		innerHTML += "<tr >";
		
		
		//Bobby:31Oct11, making messageId optional as per request from TPM
		if (this.messageId) {		
			innerHTML += "<td class=\"message-id\" tabIndex=0 aria-label=\"" + this.messageId + "\">";		
			// add URL to infocenter
			if (this.url) {
				innerHTML += '<a href="javascript:void(0)" onclick="window.open(\'' + this.url+"','"
					+ this.windowName+"','"
					+ this.windowSpecs+"','"
					+ this.windowReplace
					+ "\')\">";
				innerHTML += this.messageId;
				innerHTML += "</a>";
			}
			else {
				innerHTML += this.messageId;
			}			
			innerHTML += "</td>";
		}
		innerHTML += "</tr>";
		innerHTML += "<tr>";
		innerHTML += "<td class=\"message-description\" >";
		innerHTML += "<div id='" + this.id + "_messagediv" + "' style='overflow:auto; width:100%;' tabIndex=0 aria-label=\"" + this.message + "\">";
		innerHTML += this.message;
		innerHTML += "</div>";
		innerHTML += "</td>";
		innerHTML += "</tr>";
		if (!this.messageId) {	
			innerHTML += "<tr>";
			innerHTML += "<td>";
			innerHTML += "&nbsp;";		
			innerHTML += "</td>";
			innerHTML += "</tr>";			
		}	
		innerHTML += "</table>";

		innerHTML += "</td>";
		innerHTML += "</tr>";

		if (this.showCheckbox === true) {
			innerHTML += "<tr>";
			innerHTML += "<td>";	
			innerHTML += "</td>";			
			innerHTML += "<td class=\"checkbox\" valign=\"bottom\">";
			
			innerHTML += "<table role=\"presentation\">";
			innerHTML += "<tr valign='bottom'>";
			innerHTML += "<td>";
			innerHTML += "<input type=\"checkbox\" dojoType=\"dijit.form.CheckBox\" id=\"" + this.id + "neverShowAgain\"/>&nbsp;";
			innerHTML += "</td>";
			innerHTML += "<td valign='middle'>";
			innerHTML += "<label for=\"" + this.id + "neverShowAgain\">" + this.resources_.NEVER_SHOW + "</label>";
			innerHTML += "</td>";
			innerHTML += "</tr>";	
			innerHTML += "</table>";		
			
			innerHTML += "</td>";
			innerHTML += "</tr>";
		}

    innerHTML += "</table>";
		
		innerHTML += "</div>";
		
		return innerHTML;
	},
	
	// correctly handle PNG transparency in Win IE 5.5 & 6.
	correctPNG: function() {
		var arVersion = navigator.appVersion.split("MSIE");
		var version = parseFloat(arVersion[1]);
		if ((version >= 5.5) && document.body.filters) {
			for (var i = 0; i < document.images.length; i++) {
				var img = document.images[i];
				var imgName = img.src.toUpperCase();
				if (imgName.substring(imgName.length-3, imgName.length) == "PNG") {
					var imgID = img.id ? "id='" + img.id + "' " : "";
					var imgClass = img.className ? "class='" + img.className + "' " : "";
					var imgTitle = img.title ? "title='" + img.title + "' " : "title='" + img.alt + "' ";
					var imgStyle = "display:inline-block;" + img.style.cssText;
					if (img.align == "left") {
						imgStyle = "float:left;" + imgStyle;
					}
					if (img.align == "right") {
						imgStyle = "float:right;" + imgStyle;
					}
					if (img.parentElement.href) {
						imgStyle = "cursor:hand;" + imgStyle;
					}
					var strNewHTML = "<span " + imgID + imgClass + imgTitle +
									 " style=\"" + "width:" + img.width + "px; height:" + img.height + "px;" + imgStyle + ";" +
									 "filter:progid:DXImageTransform.Microsoft.AlphaImageLoader" +
									 "(src=\'" + img.src + "\', sizingMethod='scale');\"></span>";
					img.outerHTML = strNewHTML;
					i = i - 1;
				}
			}
		}    
	},
	
	destroy: function () {
		// summary:  destroys the message Dialog.
		// tags: public
        dojo.disconnect( this.onCloseListener );
		if(this.onResizeListener){
			dojo.disconnect( this.onResizeListener );
		}
        
        //TODO:  The fact this can NOT be called means we have not created the widget correctly
        //       This widget should directly extend dialog to be created correctly
        //this.inherited( "destroy", arguments );
        
        if ( this.dialog_ ) { this.dialog_.destroy(); dijit.registry.remove(this.dialog_.id); delete this.dialog_; }
        
		if (dijit.byId (this.id + "neverShowAgain")) {
			dijit.registry.remove (this.id + "neverShowAgain");
		}

		for (var i = 0; i < this.buttons.length; i++) {
			dijit.registry.remove (this.id + "mmd_button_" + i);
		}

		delete this.type;
		delete this.message;
		delete this.messageId;
		delete this.width;
		delete this.height;
		delete this.buttons;
		delete this.callbackParms;
		delete this.url;
		delete this.showCheckbox;
		delete this.destroyOnClose;
		delete this.title_;
		delete this.icon_;
		delete this.resources_;

		dijit.registry.remove (this.id);
	}

});



});
