/*********************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* Tivoli Widget Library
*
* (C) Copyright IBM Corp. 2013.  All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
************************************************************ {COPYRIGHT-END} **/
define("dojoe/ssd/utils/EventHelper", [
	"dojo/_base/declare",
	"ibm/tivoli/tip/dojo/EventUtil",
	"ibm/tivoli/navmodel/DatasetUtils"
], function (declare, EventUtil) {
	 return declare("dojoe.ssd.utils.EventHelper", null, {
	 
		portletNamespace: null,
		
		constructor: function (portletNamespace) {
			//lang.mixin (this, props);
			this.portletNamespace = portletNamespace;
		},

		getSpecialParams: function (datasetInfo) {
			console.log ("EventHelper.js: enter eventHelper.getSpecialParams");		
			var paramsAndMssname;
			var requiredParams = new Array();
			var providerMSSName;
			var providerType;
			try{
				//Bobby: 03Oct2012, creating a new object instead of messing up current one
				var tempDatasetInfo =  dojo.mixin({}, datasetInfo);
				ibm.tivoli.navmodel.DatasetUtils.getParameters(dojo.mixin(tempDatasetInfo,{
					sync: true, //Bobby: 03Oct2012, wont work in this scenario if async
					onComplete: function(parameters, kwArgs) {						
						for (var i=0;i<parameters.length;i++) {
							if(parameters[i].isRequired())
								requiredParams.push(parameters[i].getId());
							if(parameters[i].getId() == "MSSName")
								providerMSSName = parameters[i].getValue();
							if(parameters[i].getId() == "ProviderType")
								providerType = parameters[i].getValue();
						}					
					},
					onError: function(error, exception, ioArgs) {
						console.warn("EventHelper.js: Error occured while fetching dataset params...:"+error);
						console.error(exception);	
						return null;
					}
				}));
			} catch(ex) {
				console.error("EventHelper.js: Exception : " + ex);
			}
			paramsAndMssname = {
				reqdParams : requiredParams,
				MSSName : providerMSSName,
				ProviderType : providerType
			};	
			return paramsAndMssname;
		},

		publishNodeClickedOnEvent: function(dataset, itemId, portletNamespace) {
			console.info("EventHelper.js: eventHelper.publishNodeClickedOnEvent(): BEGIN");
			var eventURL = dataset.getAttributes()["EventURL"];
			var item = dataset.getItem({"itemId":itemId});
			if(!portletNamespace) portletNamespace = this.portletNamespace;//for backward compatibility
			console.info("EventHelper.js: portletNamespace --->"+portletNamespace);
			
			if(!portletNamespace){
				console.error("EventHelper.js: portletNamespace is missing, aborting publishNodeClickedOnEvent...");
				return;
			}
			
			//52019: TBSM 34795 - Wiring fails due to request for node that does not exist in dataset used by target widget
			//Bobby: 18October, don't pass source params to target
			/*
			//Pass all the original params & eventURL to processItemProperties
			var allParameters = dojo.mixin({},dataset.getArg('parameters'));
			//Bobby:16Oct2012, Remove dataset specific params
			delete allParameters.pageId;
			delete allParameters.refId;
			delete allParameters.tz;
			
			console.info("EventHelper.js: allParameters"+allParameters);
			console.dir(allParameters);*/
			//if (item) eventHelper.processItem(item, eventURL, allParameters);
			if (item) this.processItem(item, eventURL, portletNamespace);
			console.info("EventHelper.js: eventHelper.publishNodeClickedOnEvent: END");	
		},

		processItem: function(item, eventURL, portletNamespace){
			console.info("EventHelper.js: eventHelper.processItem(): BEGIN");	
			var kwArgs = {};
			kwArgs.onComplete = dojo.hitch(this, this.processItemProperties);
			kwArgs.onError = dojo.hitch(this, this.processError);
			//Just relay these, not used by item.getProperties
			kwArgs.eventHelper_eventURL = eventURL;
			kwArgs.eventHelper_portletNamespace = portletNamespace;
			//kwArgs.eventHelper_allParameters= allParameters;
			item.getProperties(kwArgs);
			console.info("EventHelper.js: eventHelper.processItem(): END");
		},

		processItemProperties: function(props, args)	{
			console.info("EventHelper.js: eventHelper.processItemProperties(): BEGIN");
			//console.info("props --->");
			//console.dir(dojo.mixin({},props));	
			var attributes = new Object();
			attributes.valueMap = new Array();
			var itemObj = new Object();
			var eventURIProps = this.getEventURL(props, args.eventHelper_eventURL);
			if(eventURIProps) {
				for(var prop in eventURIProps)
					itemObj[prop] = eventURIProps[prop];
			} else if(this.getResourceMap(props) !== undefined) {
				var allProps = this.getResourceMap(props);
				var resourceMap = allProps["resourceMap"];
				for(var i in allProps["resourceMap"]) {
					var val = resourceMap[i];
					val = val.substr(2, (val.length-4));
					if(allProps["valueMap"]) {
						var valueMapVal = (allProps["valueMap"])[val];
						resourceMap[i] = allProps[valueMapVal];
					} else {
						var val = resourceMap[i];
						if(val.indexOf("%%") == 0 && val.lastIndexOf("%%") > 0) {
							val = val.substr(2, (val.length-4));
							resourceMap[i] = allProps[val];
						}
							
					}
				}
				itemObj = resourceMap;
			} else if(this.getValueMap(props) !== undefined) {
				var allProps = this.getValueMap(props);
				var valueMap = allProps["valueMap"];
				for(var i in allProps["valueMap"]) {
					valueMap[i] = allProps[(valueMap[i])];
				}
				itemObj = valueMap;
			} else {
				for(var i=0;i<props.length;i++){
					//55711: Problem sending events between TWL and Charts
					//Bobby: 18Oct2012, dont send object type properties like Status	
					if(props[i].getValue().getValue)
						itemObj[props[i].getId()] = props[i].getValue().getValue();
					else itemObj[props[i].getId()] = props[i].getValue();
				}
			}
			//52019: TBSM 34795 - Wiring fails due to request for node that does not exist in dataset used by target widget
			//Bobby: 18October, don't pass source params to target
			//console.info("EventHelper.js: args.eventHelper_allParameters");
			//console.debug(args.eventHelper_allParameters);
			//dojo.mixin(itemObj, args.eventHelper_allParameters);
			//console.info("itemObj --->");
			//console.dir(itemObj);
			attributes.valueMap.push(itemObj);
			console.info("EventHelper.js: attributes --->");
			console.dir(dojo.mixin({},attributes));	
			console.info("EventHelper.js: portletNamespace --->"+args.eventHelper_portletNamespace);
			//createEvent: function(attributes,eventName)
			var eventUtil = new ibm.tivoli.tip.dojo.EventUtil();
			var eventObject = eventUtil.createEvent(attributes, "http://ibm.com/tip#NodeClickedOn");
			try{				
				EventBroker.sendEvent(args.eventHelper_portletNamespace, eventObject);
			}catch(error){
				console.error(error);
			}	
			console.info("EventHelper.js: eventHelper.processItemProperties(): END");
		},

		getEventURL: function(props, eventURL)	{
			console.log("EventHelper.js: eventUrl in getEventUrl : " + eventURL);
			var attrs;	
			var requestBody=new Object();
			var valueArray = new Array();
			var eventObject = new Object();
			eventObject["id"] = "eventName";
			eventObject["value"] = "http://ibm.com/tip#NodeClickedOn";
			valueArray.push(eventObject);
			for(var i=0;i<props.length;i++){
				if(props[i].getId() == "EventURLID") {
					var idObject = new Object();
					idObject["id"] = "ID";
					idObject["value"] = (props[i].getValue()).toString();
					valueArray.push(idObject);
					requestBody["values"] = valueArray;
					var data = dojo.toJson(requestBody);
					dojo.xhrPost ({
						url: eventURL+"?properties=all",
						headers: {
							'Accept': 'application/json',
							'Content-Type':'application/vnd.ibm.com.tivolidis.json;version=1;format=customQuery',
							'X-Method-Override':'Custom-GET',
							'Content-Length':data.length
						},
						handleAs: "json",
						postData: data,
						sync: true,			
						load: dojo.hitch(this, function(result, ioArgs) {
							var allItems = result.items;
							attrs = new Object();
							for(var item in allItems) {
								var props = allItems[item].properties;
								for(prop in props) {
									attrs[props[prop].id] = props[prop].value;
								}
							}
						}),
						error: dojo.hitch(function(error, ioArgs) {
							console.error(error);
						})
					});	
				}
			}
			return attrs;
		},

		getValueMap: function(props) {
			var allProps = new Object();
			for(var i=0;i<props.length;i++){
				allProps[props[i].getId()] = props[i].getValue();
			}
			if(allProps["valueMap"]) 
				return allProps;
			else
				return undefined;
		},

		getResourceMap: function(props)	{
			var allProps = new Object();
			for(var i=0;i<props.length;i++){
				allProps[props[i].getId()] = props[i].getValue();
			}
			if(allProps["resourceMap"]) 
				return allProps;
			else
				return undefined;
		},

		processEvent: function(event, datasetInfo, dashletConfig, dataset) {
			console.log("EventHelper.js: ProcessEvent() -- BEGIN : ");
			console.info("EventHelper.js: event:"+event);
			console.dir(event);
			console.info("EventHelper.js: current params: ");
			console.dir(dojo.mixin({},dataset.getArg('parameters')));
			//Bobby:16Oct2012, adding baseUrl check to ignore ClientDatasets
			if(!datasetInfo.provider || !datasetInfo.dataset || !datasetInfo.datasource || !datasetInfo.baseUrl) {
				console.log("EventHelper.js: Dataset is not configured, ignoring event");
				return;
			}
			var eventUtil = new ibm.tivoli.tip.dojo.EventUtil();
			//var sourceParams = new Object();	
			var sourceParams,strParams;
			//54958: TIP Eventing - Error while dashlets receive a single resource format event
			//dojo.mixin(sourceParams,event.payload.resources[0]);
			//Bobby: 10Oct2012, we need to support both single and multiple resource formats
			if(event.payload) {
				if(event.payload.resources) {
					sourceParams = event.payload.resources[0];
				} else {
					sourceParams = event.payload; //  If there is only one resource
				}
				strParams = eventUtil.attributesToStrings(sourceParams);	
				//console.info("strParams -->");
				//console.dir(strParams);
			}else {
				console.warn("EventHelper.js: Payload not found, ignoring event")
				return;
			}
			
			//Bobby: 03Oct2012, still reqd for MSSName, SourceToken etc..
			var specialParams = this.getSpecialParams(datasetInfo);
			var sourceTokenReqd = false;
			if( !specialParams) { // error occured while getting params
				console.log("EventHelper.js: Error occurred while getting the special params, ignoring event");
				return;
			}
			console.debug ("EventHelper.js: specialParams", specialParams);	
			if(specialParams.reqdParams.length == 0) {
				console.log("EventHelper.js: No special parameter found, update widget");
			} else {
				var eventAttributes = eventUtil.getAttributesWithFlatProduct(event, 0, specialParams.MSSName, specialParams.ProviderType);
				for(attr=0; attr<specialParams.reqdParams.length; attr++) {
					var val = specialParams.reqdParams[attr];
					if(val == "SourceToken") { 
						sourceTokenReqd = true;
					}
					//59773: TWL 1.1 : Eventing with demo provider is ignored ' DEPTH is required param, not found in the payload, ignore event '
					//Bobby: 08Dec2012, Don't ignore if reqd param not found. It would be already configured for reciever
					/*if(!eventAttributes[val]) {
						console.log("EventHelper.js: "+specialParams.reqdParams[attr] + " is required param, not found in the payload, ignore event")
						return;
					}*/
				}
				console.log("EventHelper.js: All special params found, continue");		
				dojo.mixin(strParams,eventAttributes);
				
				var MSSname;
				var sourceToken;
				if(sourceTokenReqd) {
					// SourceToken is a required param here.
					// Now check that each resource in event have the recipient�s provider type
					var allResources = new Array();
					if(event.payload.resources) {
						allResources = event.payload.resources;
					} else {
						allResources = event.payload; //  If there is only one resource
					}
					for(index in allResources) {
						if(allResources[index].product[requiredParams.ProviderType]) {
							var resourceData = eventUtil.getAttributesWithFlatProduct(event, index, requiredParams.MSSName, requiredParams.ProviderType);
							if(!resourceData["MSSName"]) {
								console.debug("EventHelper.js: Providers MSSName not found across resource : ", allResources[index]);
								console.log("EventHelper.js: ignoring event");
								return;
							}
						} else {	
							console.log("EventHelper.js: ProviderType did not match, ignore event");
							return;
						}
					}
				}
			}
			this.getItems(strParams, datasetInfo, dashletConfig,dataset);
			console.log("EventHelper.js: ProcessEvent() -- END : ");	
		},
		
		getItems: function(eventsParam, datasetInfo, dashletConfig, dataset) {	
			console.log("EventHelper.js: getItems() - BEGIN");
			console.info("EventHelper.js: dataset: "+dataset);
			
			try{		
				console.info('EventHelper.js: going to destroy current dataset...');
				dataset.destroy();
				console.info('EventHelper.js: destroyed current dataset...');	
				
				console.log("EventHelper.js: getItems(): old params ->"+this.objLength(dataset.getArg('initParameters')));
				console.dir(dojo.mixin({},dataset.getArg('initParameters')));			
				
				console.info("EventHelper.js: getItems(): adding params from eventsParam ->"+this.objLength(eventsParam));
				console.dir(eventsParam);			
				var newDatasetInfo = dojo.mixin({},datasetInfo);
				//Bobby: 19Oct2012, Reset to a copy of original params at the time of creation
				newDatasetInfo.parameters = dojo.mixin({},dataset.getArg('initParameters'));
				dojo.mixin(newDatasetInfo.parameters, eventsParam);
				console.log("EventHelper.js: getItems(): new params ->"+this.objLength(newDatasetInfo.parameters));
				console.info("EventHelper.js: getItems(): newDatasetInfo ->"+newDatasetInfo);
				console.dir(newDatasetInfo);
				dataset = new ibm.tivoli.navmodel.Dataset(newDatasetInfo);
			} catch (ex) {
				console.error ("EventHelper.js: Caught exception in getItems:");
				console.error (ex);
			}					
			console.log("EventHelper.js: getItems() - END");
		},	

		objLength :function(obj){
			var length = 0;
			if(typeof obj != 'object'){
				return false;
			}
			for(var i in obj) {
				length++;
			}	
			return length;
		}
		});
});
