// wrapped by build app
define("dojoe/tablex/_Toolbar", ["dijit","dojo","dojox","dojo/i18n!dojoe/tablex/nls/Table","dojo/require!dojoe/common/toolbar/_Toolbar,dijit/form/Button,dijit/ToolbarSeparator,dijit/form/DropDownButton,dijit/form/ComboButton,dijit/Menu,dijit/PopupMenuItem,dijit/Dialog,dijit/form/SimpleTextarea,dojo/i18n,dojoe/tablex/_ConfigureColumnsDialog,dojoe/tablex/_QuickFilter,dijit/_TemplatedMixin"], function(dijit,dojo,dojox){
/*********************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* Tivoli Widget Library
*
* (C) Copyright IBM Corp. 2013.  All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
************************************************************ {COPYRIGHT-END} **/
// dojo.provide allows pages to use all of the types declared in this resource.
dojo.provide("dojoe.tablex._Toolbar");

dojo.require("dojoe.common.toolbar._Toolbar");
dojo.require("dijit.form.Button");
dojo.require("dijit.ToolbarSeparator");
dojo.require("dijit.form.DropDownButton");
dojo.require("dijit.form.ComboButton");
dojo.require("dijit.Menu");
dojo.require("dijit.PopupMenuItem"); // 55027 - Defaults for Table Widget (Required for Print and Export Popup Menu)
dojo.require("dijit.Dialog");
dojo.require("dijit.form.SimpleTextarea");
dojo.require("dojo.i18n");
dojo.require("dojoe.tablex._ConfigureColumnsDialog");
dojo.require("dojoe.tablex._QuickFilter");
dojo.require("dijit._TemplatedMixin");

dojo.requireLocalization("dojoe.tablex", "Table");

dojo.declare("dojoe.tablex._Toolbar", [ dojoe.common.toolbar._Toolbar], {
		// summary:
		//		Table toolbar class which is based on common toolbar widget and is a private class
		// description:
		//		This class provides ability to enable/disable Table specific toolbar controls
		//		like refresh icon, action menu, print, export, configure columns and filter/search options
		//		this is internally used by Table class and can be configured on Table widget class
		
		// widgetsInTemplate: boolean
		//		this is used to indicate there are widgets in template
		widgetsInTemplate:true,
		
		// plugins: object
		//		array of plugins associated with the grid instance
		plugins:null,
		
		// inGrid: object
		//		Grid instance
		inGrid:null,
		
		// actionListWidget: object
		//		Action list widget instance
		actionListWidget:null,
		
		// resources reference
		resources_:null,
		
		// toolbarOptions from table
		toolbarOptions:null,
		
		// filterOptions: object
		//		To hide/show filter bar, to hide/show Quick Filter, hide/show Advnaced filter
		filterOptions:null,
		
		// refreshIcon: boolean
		//		To enable/disable refresh icon on toolbar, defaults to true 
		refreshIcon:true,
		
		// refreshMenu: boolean
		//		To enable/disable refresh item in action list, defaults to true 
		refreshMenu:true,
		
		// actionMenu: boolean
		//		To enable/disable action list in the toolbar, defaults to true 
		actionMenu:true,
		
		// 55027 - Defaults for Table Widget
		// exporterMenu: boolean
		//		To indicate whether export options should appear in action list, defaults to true
		exporterMenu:true,
		
		// 55027 - Defaults for Table Widget
		// printerMenu: boolean
		//		To indicate whether print options should appear in action list, defaults to true
		printerMenu:true,
		
		// configTableIcon: boolean
		//		To enable/disable configure options Icon in the toolbar, defaults to true
		configTableIcon:false, // 55027 - Defaults for Table Widget
		
		// configTableMenu: boolean
		//		To indicate whether configure options should appear in action list, defautls to true
		configTableMenu:true,
		
		//table instance
		tivTable:null,
		
		// configure dialog isntance
		configureTableDialog:null,
		
		// just the quickFilter instance
		_quickFilter:null,

		_origStructure: null,
		
		_formattersArr: null,
		
		// Constructor function. Called when instance of this class is created
		constructor : function() {
			this.resources_ = dojo.i18n.getLocalization("dojoe.tablex", "Table", this.lang);			
		},
 		
		postCreate:function(){		
			this.inGrid = this.tivTable._grid;
			this._origStructure = this.tivTable._origLayout;
			this.plugins = this.tivTable.gridParams_.plugins;
			this.toolbarOptions = this.tivTable.toolbarOptions;
			this.filterOptions = this.tivTable.filterOptions;
			//commenting as this not required
			//this.toolbar = new dijit.Toolbar({id: "toolbar"+this.id});			
			
			// 53654 : Support for mobile mode
			if(this.tivTable.mobileMode) {
				this.filterOptions.quickFilter = true;
				this.filterOptions.showQuickFilterButton = false;
				this.filterOptions.advancedFilter = false;
			}			
			
			//setting the toolbar options passed from UI
			this._mapToolbarOptions();			
			
			//adding the refresh icon
			this._addRefreshIcon();			
			
			// 53654 : Support for mobile mode
			if (!this.tivTable.mobileMode) {
			
				//adding the export icon
				this._addExporterIcon();
						
				//adding the printer icon
				this._addPrintIcon();				
			
				//adding the configure settings icon
				this._addConfigTableIcon();				
			
				//Action List Menu
				this._addActionMenu();			
			}
			
			//adding filters
			this._addFilters();
			
		},
		
		//16716: table toolbar has browser popup menu on right click
		_stopEvent: function(e){
			// summary:
			//		dojo stop event
			//Since event is oncontextmenu, just stop it!
			dojo.stopEvent(e);
		},
		
		
		_addRefreshIcon: function() {
			// summary:
			//		internal function to add refresh icon to the table toolbar
			
			if(this.refreshIcon) {
				var refreshTableIcon = new dijit.form.Button({
						showLabel: false,
						id: "refreshIcon"+this.id,
						iconClass: "refreshIcon",
						label: this.resources_.REFRESH,
						title: this.resources_.REFRESH
					});
					
				refreshTableIcon.onClick = dojo.hitch (this, "_refreshTable");
				refreshTableIcon.onKeypress = dojo.hitch (this, "_refreshTable");
				this._addStateControl(refreshTableIcon,0);
				
				// 53654 : Support for mobile mode
				if (!this.tivTable.mobileMode) {
					var seperator = new dijit.ToolbarSeparator({id: "refSeperator"+this.id});
					this._addStateControl(seperator);
				}
			}
		},
		_addExporterIcon: function() {
			// summary:
			//		internal function to add Export icon to the table toolbar
			
			// 55027 - Defaults for Table Widget
			if(this.toolbarOptions && this.toolbarOptions.exporterIcon) {
				var exportDataMenu = new dijit.Menu({ style: "display: none;"});
				var exportHTMLMenuItem = new dijit.MenuItem({
					title: this.resources_.EXPORT_HTML,
					label: this.resources_.EXPORT_HTML
				});
				exportHTMLMenuItem.onClick = dojo.hitch (this, "_exportHTML");
				exportHTMLMenuItem.onKeypress = dojo.hitch (this, "_exportHTML");
				exportDataMenu.addChild(exportHTMLMenuItem);	
				
				//Bobby: 07Jan2011, adding CSV Export 
				var exportCSVMenuItem = new dijit.MenuItem({
					title: this.resources_.EXPORT_ALL_CSV,
					label: this.resources_.EXPORT_ALL_CSV
				});
				exportCSVMenuItem.onClick = dojo.hitch (this, "_exportCSV");
				exportCSVMenuItem.onKeypress = dojo.hitch (this, "_exportCSV");
				exportDataMenu.addChild(exportCSVMenuItem);				
			
				//Combo Button for All Export Related Actions
				var exportDataCombo = new dijit.form.DropDownButton({
					showLabel: false,
					iconClass: "exportDataIcon",
					id: "exportDataIcon"+this.id,
					label: this.resources_.EXPORT,
					title: this.resources_.EXPORT,
					dropDown: exportDataMenu
				});
				this._addFastPathControl(exportDataCombo);
			}
		},
		
		_addPrintIcon: function() {
			// summary:
			//		internal function to add print icon to the table toolbar
			
			// 55027 - Defaults for Table Widget
			//Check whether we need to show the print icon or not
			if( this.toolbarOptions && this.toolbarOptions.printerIcon) {
				var printDataMenu = new dijit.Menu({ style: "display: none;"});		    
				var printAllItem = new dijit.MenuItem({
					title: this.resources_.PRINT_ALL,
					label: this.resources_.PRINT_ALL
				});
				printAllItem.onClick = dojo.hitch (this, "_printAll");
				printAllItem.onKeypress = dojo.hitch (this, "_printAll");			     
				printDataMenu.addChild(printAllItem);
				var printSelectedMenuItem = new dijit.MenuItem({
					title: this.resources_.PRINT_SELECTED,
					label: this.resources_.PRINT_SELECTED
				});
				printSelectedMenuItem.onClick = dojo.hitch (this, "_printSelected");
				printSelectedMenuItem.onKeypress = dojo.hitch (this, "_printSelected");
				printDataMenu.addChild(printSelectedMenuItem);		     
				var printPreviewMenuItem = new dijit.MenuItem({
					title: this.resources_.PRINT_PREVIEW,
					label: this.resources_.PRINT_PREVIEW
				});
				printPreviewMenuItem.onClick = dojo.hitch (this, "_printPreview");
				printPreviewMenuItem.onKeypress = dojo.hitch (this, "_printPreview");
				
				printDataMenu.addChild(printPreviewMenuItem);		     
				//Combo Button for All Print Related Actions
				var printDataCombo = new dijit.form.DropDownButton({
					showLabel: false,
					iconClass: "printIcon",
					id: "printIcon"+this.id,
					label: this.resources_.PRINT,
					title: this.resources_.PRINT,
					dropDown: printDataMenu
				});
				this._addFastPathControl(printDataCombo);
			}
		},
		_addConfigTableIcon: function() {
			// summary:
			//		internal function to add config icon to the table toolbar
			
			if(this.configTableIcon) {
				var configTableIcon = new dijit.form.Button({
						showLabel: false,
						iconClass: "configureColIcon",
						id: "configureColIcon"+this.id,
						label: this.resources_.CONFIGURE_SETTINGS,
						title: this.resources_.CONFIGURE_SETTINGS
					});
					
				configTableIcon.onClick = dojo.hitch (this, "_configureTable");
				configTableIcon.onKeypress = dojo.hitch (this, "_configureTable");
				this._addFastPathControl(configTableIcon);
			}
		},
		
		_addActionMenu: function() {
			// summary:
			//		internal function to add actionlist icon to the table toolbar
			
			//Action List Menu
			if(this.actionMenu) {			
				//var seperator = new dijit.ToolbarSeparator({id: "actionMenuSeperator"+this.id},this.actionSeperator);
				var menu = new dijit.Menu({ style: "display: none;"});
				//menu.addChild(new dijit.MenuSeparator());
				
				//23789: Refresh should be part of Actions Menu also - OID
				//Bobby:29Jun11				
				if(this.refreshMenu) {
					//Menu Item for Configure Columns action
					var refreshMenuItem = new dijit.MenuItem({
						//Bobby:19Jul11, title is reqd by JAWS to anounce the menu item.					
						//24709: JAWS does not read the items in the Actions, Export or Print menus.
						title: this.resources_.REFRESH,
						label: this.resources_.REFRESH,
						iconClass: "refreshIcon"
					});
					refreshMenuItem.onClick = dojo.hitch (this, "_refreshTable");
					refreshMenuItem.onKeypress = dojo.hitch (this, "_refreshTable");					 
					menu.addChild(refreshMenuItem);
				}

				if(this.filterOptions.showAdvancedFilter){
					//Menu Item for Advanced Filter action
					var filterMenuItem = new dijit.MenuItem({
						title: this.resources_.ADVANCED_FILTER,
						label: this.resources_.ADVANCED_FILTER,
						//20508: Advanced Filter menu item on Actions menu has tooltip while other menu items do not
						//Bobby:25May11, removing unnecessary title, not reqd for Menu
						//title: this.resources_.ADVANCED_FILTER,
						iconClass: "filterIcon"
					});
					filterMenuItem.onClick = dojo.hitch (this, "_advancedFilter");
					filterMenuItem.onKeypress = dojo.hitch (this, "_advancedFilter");					 
					menu.addChild(filterMenuItem);
				}				
				
				// 55027 - Defaults for Table Widget
				if(this.exporterMenu) {
					//Sub-menu item for All Export Related Actions
					var exportDataSubMenu = new dijit.Menu();
					exportDataSubMenu.addChild(new dijit.MenuItem({
						title: this.resources_.EXPORT_HTML,
						label: this.resources_.EXPORT_HTML,
						onClick: dojo.hitch (this, this._exportHTML),
						onKeypress: dojo.hitch (this, this._exportHTML)
					}));	
					
					//Bobby: 07Jan2011, Adding export as CSV
					exportDataSubMenu.addChild(new dijit.MenuItem({
						title: this.resources_.EXPORT_ALL_CSV,
						label: this.resources_.EXPORT_ALL_CSV,
						onClick: dojo.hitch (this, this._exportCSV),
						onKeypress: dojo.hitch (this, this._exportCSV)
					}));					
						
					var exportDataItem = new dijit.PopupMenuItem({
						title: this.resources_.EXPORT,
						label: this.resources_.EXPORT,
						iconClass: "exportDataIcon",
						popup: exportDataSubMenu
					});
				    menu.addChild(exportDataItem);
				}
				
				// 55027 - Defaults for Table Widget
				if(this.printerMenu) {
					//Sub-menu items for All Print Related Actions
					var printDataSubMenu = new dijit.Menu();
					printDataSubMenu.addChild(new dijit.MenuItem({
						title: this.resources_.PRINT_ALL,
						label: this.resources_.PRINT_ALL,
						onClick: dojo.hitch (this, this._printAll),
						onKeypress: dojo.hitch (this, this._printAll)
					}));
					printDataSubMenu.addChild(new dijit.MenuItem({
						title: this.resources_.PRINT_SELECTED,
						label: this.resources_.PRINT_SELECTED,
						onClick: dojo.hitch (this, this._printSelected),
						onKeypress: dojo.hitch (this, this._printSelected)
					}));
					printDataSubMenu.addChild(new dijit.MenuItem({
						title: this.resources_.PRINT_PREVIEW,
						label: this.resources_.PRINT_PREVIEW,
						onClick: dojo.hitch (this, this._printPreview),
						onKeypress: dojo.hitch (this, this._printPreview)
					}));
					var printMenuItem = new dijit.PopupMenuItem({
						title: this.resources_.PRINT,
						label: this.resources_.PRINT,
						iconClass: "printIcon",
						popup: printDataSubMenu
					});
					menu.addChild(printMenuItem);
				}
				
				if(this.configTableMenu) {
					//Menu Item for Configure Columns action
					var configureColMenuItem = new dijit.MenuItem({
						title: this.resources_.CONFIGURE_SETTINGS,
						label: this.resources_.CONFIGURE_SETTINGS,
						iconClass: "configureColIcon"
					});
					configureColMenuItem.onClick = dojo.hitch (this, "_configureTable");
					configureColMenuItem.onKeypress = dojo.hitch (this, "_configureTable");					 
					menu.addChild(configureColMenuItem);
				}				
				
			    menu.startup();
				//Initializing the Action List
			    var actionListMenu = new dijit.form.DropDownButton({
					optionsTitle : this.resources_.ACTIONS,
			        label: this.resources_.ACTIONS,
					title: this.resources_.ACTIONS,
					id: "actionListMenu"+this.id,
			        dropDown: menu
			    });				
				this.actionListWidget = actionListMenu;
				this._addFastPathControl(actionListMenu);
			}
			
		},
		
		// To create quick filter
		_createQuickFilter: function() {
			// summary:
			//		internal function to create quick filter
			this._quickFilter = new dojoe.tablex._QuickFilter({
						grid:this.inGrid,
						id: "quickFilter"+this.id,
						filterOptions: this.filterOptions,
						placeHolder: this.resources_.QF_DEFAULT_TEXT,
						clearIconText:this.resources_.CLEAR_FILTER
			}, this.quickFilterNode);
		},
		
		// To create advanced filter
		_createAdvanceFilter: function() {
			// summary:
			//		internal function to create advance filter 
			
			var advancedFilterIcon = new dijit.form.Button({
				showLabel: false,
				iconClass: "filterIcon",
				id: "advancedFilterIcon"+this.id,
				//label: this.resources_.ADVANCED_FILTER,
				title: this.resources_.ADVANCED_FILTER
			},this.filterButton);				
			advancedFilterIcon.onClick = dojo.hitch (this, "_advancedFilter");
			advancedFilterIcon.onKeypress = dojo.hitch (this, "_advancedFilter");
			//advancedFilterIcon.domNode.title = this.resources_.ADVANCED_FILTER;
		},
		
		_addFilters: function() {
			// summary:
			//		internal function to add filter options to the toolbar
			
			//adding filters
			if(this.filterOptions.hideFilterBar == true && this.inGrid.filterBar !== undefined){
				//hide Advanced Filter bar
				this.inGrid.filterBar.hide();
			} else if(this.filterOptions.hideFilterBar == false){
				//show Advanced Filter bar
				this.inGrid.filterBar.show();
			}			
			
			/*If 
				quickFilter 		- YES
				quickFilterButton 	- YES
				advancedFilter		- NO
			  then show only quickFilterButton
			*/
			if(this.filterOptions.quickFilter && this.filterOptions.showQuickFilterButton && !this.filterOptions.advancedFilter){
				this._createQuickFilter();
				var quickFilterIcon = new dijit.form.Button({
						showLabel: false,
						iconClass: "filterIcon",
						id: "filterIcon"+this.id,
						//label: this.resources_.QUICK_FILTER,
						title: this.resources_.QUICK_FILTER
					},this.filterButton);				
				quickFilterIcon.onClick = dojo.hitch (this, "_setQuickFilter");
				quickFilterIcon.onKeypress = dojo.hitch (this, "_setQuickFilter");
				quickFilterIcon.domNode.title = this.resources_.QUICK_FILTER;				
			}			
			/*If 
				quickFilter 		- YES
				quickFilterButton 	- YES
				advancedFilter		- YES
			  then show filter combobutton
			*/			
			else if(this.filterOptions.quickFilter && this.filterOptions.showQuickFilterButton && this.filterOptions.advancedFilter){
				this._createQuickFilter();
				var menu = new dijit.Menu({ });
				menu.domNode.style.display="none";
				var qfMenuItem = new dijit.MenuItem({
					label: this.resources_.QF_DEFAULT_TEXT,
					//iconClass: "filterIcon",
					onClick: dojo.hitch (this, "_setQuickFilter"),
					onKeypress: dojo.hitch (this, "_setQuickFilter")
				});
				var afMenuItem = new dijit.MenuItem({
					label: this.resources_.ADVANCED_FILTER,
					//iconClass: "filterIcon",
					onClick: dojo.hitch (this, "_advancedFilter"),
					onKeypress: dojo.hitch (this, "_advancedFilter")
				});				
				menu.addChild(qfMenuItem);
				menu.addChild(afMenuItem);
				
				var comboFilterIcon = new dijit.form.ComboButton({
						showLabel: false,
						iconClass: "filterIcon",
						id: "comboFilterIcon"+this.id,
						label: this.resources_.QUICK_FILTER,
						title: this.resources_.QUICK_FILTER,
						dropDown: menu,
						onClick: dojo.hitch (this, "_setQuickFilter"),
						onKeypress: dojo.hitch (this, "_setQuickFilter")
					},this.filterButton);
			}
			/*If 
				quickFilter 		- YES
				quickFilterButton 	- NO
				advancedFilter		- NO
			  then show filter textbox only
			*/				
			else if(this.filterOptions.quickFilter && !this.filterOptions.showQuickFilterButton && !this.filterOptions.advancedFilter){
				this._createQuickFilter();
			}
			/*If 
				quickFilter 		- YES
				quickFilterButton 	- NO
				advancedFilter		- YES
			  then show filter textbox and AdvancedFilter button
			*/			
			
			else if(this.filterOptions.advancedFilter && !this.filterOptions.showQuickFilterButton && this.filterOptions.advancedFilter){
				this._createQuickFilter();
				this._createAdvanceFilter();
			}
			/*If 
				quickFilter 		- NO
				quickFilterButton 	- NO
				advancedFilter		- YES
			  then show AdvancedFilter button only
			*/				
			else if(this.filterOptions.advancedFilter) {
				this._createAdvanceFilter();					
			}//else don't show qf or af.
			
		},
		
		_refreshTable: function(){
			// summary:
			//		internal function to refresh table
			this.tivTable.refresh();
		},
		_populateFormatterArr: function(){
			if(this.formattersArr == null && this.tivTable.defaultExportFormatter == false){
				this._formattersArr = new Array();
			
				var colsById = this.inGrid._columnsById;
				if(this._formattersArr.length == 0 && this.tivTable.exportFormatter != null){
		            for(colId in colsById){
		                //popuate the _formattersArr with a funtion that formats data from properties
						//to ensure data is consistent in the exporter/printed page with table cells
		                this._formattersArr[colId] = dojo.hitch (this.tivTable, "exportFormatter");//this.tivTable.exportFormatter;
		                	
		                	
		            }
				}
			}
		},
		
			
		_exportHTML: function(){
			var url = dojo.moduleUrl("dojoe", "table/themes/print_style.css");
			var urlStr = url.toString();
			
			var printArgs = {title: this.resources_.EXPORT_HTML, styleSrc:urlStr};
			this._populateFormatterArr();
            printArgs.formatters = this._formattersArr;
			if(this.inGrid.select.row.getSelected() > 0) {
				printArgs.selectedOnly = true;
				var deferred = this.tivTable._printPreviewGrid( printArgs);
				deferred.addCallback(function(exportedString) {
					var win = window.open();
					win.document.open();
					win.document.write(exportedString);
					//Do adjustments to fix the problems of preview here.					
					//grid.normalizePrintedGrid(win.document);
					win.document.close();
				});
			}
			else {			
				var deferred = this.tivTable._printPreviewGrid(printArgs);
				deferred.addCallback(
					function(str){
						var win = window.open();
						win.document.open();
						win.document.write(str);
						//Do adjustments to fix the problems of preview here.
						//grid.normalizePrintedGrid(win.document);
						win.document.close();
					}
				);
			}
		},		
		
	   //Bobby: 07Jan2011, adding CSV Export 
		_exportCSV: function(){
		 // this._customizeData();	
			var printArgs = {};

			this._populateFormatterArr();
            printArgs.formatters = this._formattersArr;
		  var deferred = this.tivTable._exportGrid("csv", printArgs);
		  deferred.addCallback(
			function(str){
				//Bobby:16Apr2011. Browser messes up the format of CSV
			    //16850: TBSM: Table CSV Export not usable by Excel
				//var win = window.open();
				//win.document.open();
				//win.document.write(str);
				//win.document.close();
				resources_ = dojo.i18n.getLocalization("dojoe.tablex", "Table", this.lang);		
				
				//Ramachandran - 26 Sept 2012 - #50135: TBSM34363: OSDP M5:Cannot dismiss Export All to CSV dlg on Android
				//Adding close button to close the dialog 
				//Adding the button and TextArea in html table for better alignment
				var divTable = dojo.create('table', {style: 'width:100%; height: 100%'});
				var divRow1 = dojo.create('tr', {}, divTable);
				var divCell1 = dojo.create('td', {}, divRow1);
				
				var divRow2 = dojo.create('tr', {}, divTable);
				var divCell2 = dojo.create('td', {style: 'text-align: right;'}, divRow2);

				var exportTextArea = new dijit.form.SimpleTextarea({
					value: str,
					rows: 20,
					cols: 100,
					//style: "width:80%; height:80%",
					selectOnClick: true
				});
				exportTextArea.selected = true;
				exportTextArea.placeAt(divCell1);
				
				//Close button to close the dialog
				var dlgCloseButton = new dijit.form.Button({label: this.resources_.DIALOG_CLOSE_BUTTON});
				dojo.connect(dlgCloseButton, 'onClick', function() {
					exportDialog.destroy();
				});
				dlgCloseButton.placeAt(divCell2);	
				
				var exportDialog = new dijit.Dialog({
					title: resources_.EXPORT_ALL_CSV+ " - " + resources_.EXPORT_ALL_CSV_SAVE
				});	
				exportDialog.attr("content", divTable);				
				exportDialog.show();				
			}
		  );
		},			
		
		_printAll: function(){
			//this._customizeData();
			/*#24000 - 11 Jul 2011 -dojo grid is now accepting only string uri for loading .css, now sure why */
			var url = dojo.moduleUrl("dojoe", "table/themes/print_style.css");
			var urlStr = url.toString();
			var printArgs = {title: this.resources_.PRINT_ALL, styleSrc: urlStr};
        
			this._populateFormatterArr();
                    printArgs.formatters = this._formattersArr;
			
			//this.tivTable.printGrid(printArgs);
			var deferred = this.tivTable._printPreviewGrid(printArgs);
			deferred.addCallback(function(exportedString) {
				var win = window.open();
				win.document.open();
				win.document.write(exportedString);
				//Do adjustments to fix the problems of preview here.					
				//grid.normalizePrintedGrid(win.document);
				win.print();
				win.document.close();
			});
			
		},		
		_printSelected: function(){
			//this._customizeData();
			/*#24000 - 11 Jul 2011 -dojo grid is now accepting only string uri for loading .css, now sure why */
			var url = dojo.moduleUrl("dojoe", "table/themes/print_style.css");
			var urlStr = url.toString();
			var printArgs = {selectedOnly: true, title: this.resources_.PRINT_SELECTED, styleSrc: urlStr};
	
			this._populateFormatterArr();
           printArgs.formatters = this._formattersArr;
			
			//this.tivTable.printGrid(printArgs);
			var deferred = this.tivTable._printPreviewGrid(printArgs);
			deferred.addCallback(function(exportedString) {
				var win = window.open();
				win.document.open();
				win.document.write(exportedString);
				//Do adjustments to fix the problems of preview here.					
				//grid.normalizePrintedGrid(win.document);
				win.print();
				win.document.close();
			});
			
		},		
		_printPreview: function(){
			//this._customizeData();
			var url = dojo.moduleUrl("dojoe", "table/themes/print_style.css");
			var urlStr = url.toString();
			var printArgs = {title: this.resources_.PRINT_PREVIEW, styleSrc: urlStr};

			this._populateFormatterArr();
           printArgs.formatters = this._formattersArr;
			if(this.inGrid.select.row.getSelected() > 0) {
				printArgs.selectedOnly = true;
				var deferred = this.tivTable._printPreviewGrid(printArgs);
				deferred.addCallback(function(exportedString) {
					var win = window.open();
					win.document.open();
					win.document.write(exportedString);
					//Do adjustments to fix the problems of preview here.					
					//grid.normalizePrintedGrid(win.document);
					win.document.close();
				});
			}
			else {			
				var deferred = this.tivTable._printPreviewGrid(printArgs);
				deferred.then(function(exportedString) {
					var win = window.open();
					win.document.open();
					win.document.write(exportedString);
					//Do adjustments to fix the problems of preview here.					
					//grid.normalizePrintedGrid(win.document);
					win.document.close();
				});
			}
		},
		_setQuickFilter: function(){
			//console.info("In _setQuickFilter.....");
			this._quickFilter._setFilter();
		},
		_advancedFilter: function(){
		    this.inGrid.filterBar.show();
		    this.inGrid.filterBar.showFilterDialog(); 
			
		},
		_configureTable: function(){
			/* 42703: TableView: Configure Options settings are not saved after refresh
			 * use table id instead of grid id to persist the grid options*/
			this.configureTableDialog = new dojoe.tablex._ConfigureColumnsDialog(
				{ grid: this.inGrid , _structure: this._origStructure, _table: this.tivTable}
			);	
			this.configureTableDialog.show();
		},
		
		//do not call this method directly, use table.addToToolbar(item,position,row) method to add a custom item to the toolbar before default buttons
		_addCustomControl: function(item, position,row){
			this.inherited(arguments);
		},		
		
		//do not call this method directly, use table.removeItemFromToolbar(itemId) method to remove a custom item from the toolbar
		_removeItem: function(itemId){
			var button= dijit.byId(itemId);
			if(button !== null && button !== undefined) {		
				if (button.domNode.parentNode) {
					//remove td which is the parent of the button
					buttonParent = button.domNode.parentNode;
					buttonParent.parentNode.removeChild(buttonParent);
				}
				button.destroyRecursive();
			}
		},
			
		_mapToolbarOptions: function(){
			if(this.toolbarOptions) {
				if(this.toolbarOptions.refreshIcon == false){
					this.refreshIcon = this.toolbarOptions.refreshIcon;
				}
				//23789: Refresh should be part of Actions Menu also - OID
				//Bobby:29Jun11
				if(this.toolbarOptions.refreshMenu == false){
					this.refreshMenu = this.toolbarOptions.refreshMenu;
				}	
				//62002: CONFIG_TABLE toolbar icon not showing up
				//Bobby: 08Jan2012, configTableIcon default is now false.				
				//if(this.toolbarOptions.configTableIcon == false){
				if(this.toolbarOptions.configTableIcon == true){
					this.configTableIcon = this.toolbarOptions.configTableIcon;
				}
				if(this.toolbarOptions.configTableMenu == false){
					this.configTableMenu = this.toolbarOptions.configTableMenu;
				}
				if(this.toolbarOptions.actionMenu == false){
					this.actionMenu = this.toolbarOptions.actionMenu;
				}
				if(this.toolbarOptions.exporterMenu == false){
					this.exporterMenu = this.toolbarOptions.exporterMenu;
				}
				if(this.toolbarOptions.printerMenu == false){
					this.printerMenu = this.toolbarOptions.printerMenu;
				}
			}
		},
		
		_addToActionList: function(item) {
			if(this.actionMenu) {
				//always add new menu items to the top of the menu
				this.actionListWidget.dropDown.addChild(item, 0);
			}
		},
		
		_removeFromActionList: function (menuItemId){		
			var menuItem = dijit.byId(menuItemId);
			if(menuItem !== null && menuItem !== undefined) {
				menuItem.destroyRecursive();
			}
		},
		
		_enableMenuItem : function(menuItemId){
			var menuItem= dijit.byId(menuItemId);
			if(menuItem !== null && menuItem !== undefined) {
				menuItem.set('disabled', false);
			}
		},
		
		_disableMenuItem : function(menuItemId){
			var menuItem= dijit.byId(menuItemId);
			if(menuItem !== null && menuItem !== undefined) {
				menuItem.set('disabled', true);
			}
		},
		
		destroy: function(){
			// summary:  Destroys the toolbar.
			// tags: public	
			//memory leak issue: custom destroy method to take care of removing all widgets					
			try{
				delete this.resources_;
				//this is for destroying child templated widgets 
				//this should happen by itself but need to figure out why dijit._Templated is not taking care of it...
				var widgets = dijit.findWidgets(this.domNode);
				dojo.forEach(widgets, function(w) {
					if(w){
						if(w.destroyRecursive){
							w.destroyRecursive();
						} else if(w.destroy){
							w.destroy();
						}
					}
				});
				// this is for widgets added as child to this widget and their children..
				//not present in template
				var widgetArr = this.getChildren();
				dojo.forEach(widgetArr, function (child) {
					if(child){
						if(child.destroyRecursive){
							child.destroyRecursive();
						} else if(child.destroy){
							child.destroy();
						}
					}
				});
				this.inherited(arguments);
			}catch(ex){
				console.warn("Widget destroy failed: "+this.id+", ex="+ex.name+":"+ex.message);
			}
		}
		
	});

});
