/*********************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* Tivoli Widget Library
*
* (C) Copyright IBM Corp. 2013.  All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
************************************************************ {COPYRIGHT-END} **/
define("dojoe/text/ssd/ToggleDirection", [
	"dojo/_base/declare", // declare
	"dojo/dom-style", // domStyle.getComputedStyle
	"dojo/_base/lang", // lang.hitch
	"dojo/dom-construct",
	"dojo/dom-attr",
	"dojo/dom-class",
	"dijit/_editor/_Plugin",
	"dijit/_editor/plugins/ToggleDir"
	//"../../form/ToggleButton"
], function(declare, domStyle, lang, domConstruct, domAttr, domClass, _Plugin, ToggleDir){

	var ToggleDirection = declare("dojoe.text.ssd.ToggleDirection", ToggleDir, {
		useDefaultCommand: false,
		command: "toggleDir",
		_DEFAULT: null,
		_LTR: 'ltr',
		_RTL: 'rtl',

		_changDir: function(dir){
			// summary:
			//		Handler for button click events, to switch the text direction of the editor
			if (dir != this._DEFAULT)
				domAttr.set(this.editor.editNode,"dir",dir);
			else
				domAttr.remove(this.editor.editNode,"dir");
			this.editor._currentDir = dir;
			this.button.set ("iconClass", this._computeIconClass(dir));
		},

		_initButton: function(){
			var bMenu = new dijit.Menu({});
			bMenu.addChild (new dijit.MenuItem ({label: "Default", iconClass: "dijitEditorIconBidi dijitEditorIconDefaultDir", onClick:dojo.hitch(this, "_changDir", this._DEFAULT)}));
			bMenu.addChild (new dijit.MenuItem ({label: "RightToLeft", iconClass: "dijitEditorIcon dijitEditorIconRightToLeft", onClick:dojo.hitch(this, "_changDir", this._RTL)}));
			bMenu.addChild (new dijit.MenuItem ({label: "LeftToRight", iconClass: "dijitEditorIcon dijitEditorIconLeftToRight", onClick:dojo.hitch(this, "_changDir", this._LTR)}));
			var dir = domAttr.get(this.editor,"dir");
			this.button = new dijit.form.DropDownButton(dojo.mixin({
				label: this.getLabel(this.command),
				dropDown: bMenu,
				showLabel: false,
				iconClass: this._computeIconClass(dir)
			}, this.params || {}));
			this.editor.onLoadDeferred.addCallback(lang.hitch(this, function(){
				var dir = domAttr.get(this.editor.editNode,"dir");
				this.editor._currentDir = dir;
				this.button.set ("iconClass", this._computeIconClass(dir));
			}));
		},
		
		setEditor: function(/*dijit.Editor*/ editor){
			this.inherited(arguments);
			var preDomFilterKeepDir = lang.hitch(this, function(node){
				if (node.firstChild && domClass.contains(node.firstChild,"TextSkin")){
					domAttr.set(node,"dir",domStyle.get(node.firstChild,"direction"));
					node.innerHTML = node.firstChild.innerHTML;
				}
				return node;
			});
			this.editor.contentDomPreFilters.push(preDomFilterKeepDir);			
		},
		_computeIconClass: function(dir){
			var iconClassVar = "dijitEditorIconDefaultDir"
			if (dir == "ltr")
				iconClassVar = "dijitEditorIconLeftToRight";
			else if (dir == "rtl")
				iconClassVar = "dijitEditorIconRightToLeft";
			return ((dir==null || dir=="") ? ("dijitEditorIconBidi " + iconClassVar) : ("dijitEditorIcon " + iconClassVar));
	
		}
	});

	// Register this plugin.
	_Plugin.registry["toggleDir"] = function(){
		return new ToggleDirection();
	};

	return ToggleDirection;
});
