// wrapped by build app
define("dojoe/wizard/WizardToCSlider", ["dijit","dojo","dojox","dojo/require!dojoe/wizard/Wizard,dijit/_WidgetBase"], function(dijit,dojo,dojox){
/*********************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* Tivoli Widget Library
*
* (C) Copyright IBM Corp. 2013.  All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
************************************************************ {COPYRIGHT-END} **/
dojo.provide("dojoe.wizard.WizardToCSlider");

dojo.require("dojoe.wizard.Wizard");
dojo.require("dijit._WidgetBase");

/**
 * Slider support for horizontal navigation in Wizard.
 */
 
dojo.declare("dojoe.wizard.WizardToCSlider", [dijit._WidgetBase], 
{
	toc: null,        // ordered list for ToC
	tocSteps: null,   // div containing toc main steps
	ltr: true, 
	tocStepsMargin: 10,
	tocMarginLeft: 2,
	oldTocStepsWidth: 0,
	totalWidthToCItems: 0,
	oneButtonWidth: 29,
	visibleButtonWidth: 0,  // width plus margin of visible buttons 
	sliderOuterDiv: null,
	sliderLeft: null,
	sliderRight: null,
	sliderLeftImg: dojo.moduleUrl("dojoe.common", "themes/claro/images/slider_left.png"),
	sliderRightImg: dojo.moduleUrl("dojoe.common", "themes/claro/images/slider_right.png"),
	t: null,
	slideDelay: 2,  // 2 milliseconds
	slidePixel: 2,
	pos: 0,
	slideDirection: "",
	slideRightLabel: "",
	slideLeftLabel: "",
  
		/**
	* Constructor which adds nodes to ToC content pane for the slider, e.g.
	* <div for main steps - passed in>
	*   <img for left slider button />
	*   <div for containing ToC (outer) >
	*     <ol for ToC (inner) - passed in />
	*   </div>
	*   <img for right slider button>
	* </div>
	* 
	* Takes three arguments:
	*   toc       <ol> for ToC
	*   tocSteps  div for main steps
	*   ltr       flag indicating whether container text direction is left-to-right
	*             or not (right-to-left)
	*/
	constructor: function (args) {
		this.toc = args.toc;
		this.tocSteps = args.tocSteps;
		this.ltr = args.ltr;
		// strings for slider buttons
		this.resources = dojo.i18n.getLocalization("dojoe.wizard", "Wizard", this.lang);
		this.slideRightLabel = this.resources.NEXT;
		this.slideLeftLabel = this.resources.PREVIOUS;
		this.sliderOuterDiv = dojo.create("div", { className: "slider-outer" });          
		dojo.place(this.toc, this.sliderOuterDiv, 0);
		dojo.addClass(this.toc, "slider-inner");        
	
		this.sliderLeft = dojo.create("img", {
			src: this._blankGif || this.sliderLeftImg.toString(),
			alt: this.slideLeftLabel,
			title: this.slideLeftLabel,
			className: "slider-left"
		});
		this.sliderRight = dojo.create("img", {
			src: this._blankGif || this.sliderRightImg.toString(),
			alt: this.slideRightLabel,
			title: this.slideRightLabel,
			className: "slider-right"
		});
		dojo.place(this.sliderLeft, this.tocSteps, 0);
		dojo.place(this.sliderOuterDiv, this.tocSteps, 1);
		dojo.place(this.sliderRight, this.tocSteps, 2);
		// initially hide buttons
		dojo.style(this.sliderLeft, "display", "none");        
		dojo.style(this.sliderRight, "display", "none");        
		// connect button handlers
		var slideLeftListener = dojo.connect(this.sliderLeft, "onclick", this, this.slideLeft);
		var slideRightListener = dojo.connect(this.sliderRight, "onclick", this, this.slideRight);
	},
  
	/**
	* Re-calculate total width of all ToC items (step arrows).
	*/
	recalcToCWidth: function () {
		// With IE arrow image parts may wrap as the arrow styles change.  
		// To ensure we have the correct total width, we temporarily increase
		// the width of the ToC list to prevent any truncation of parts and 
		// then add up the widths of the arrow parts, i.e. front, middle and 
		// tail.
		dojo.style(this.toc, "width", (this.totalWidthToCItems * 2) + "px");  // should be sufficient   
		this.totalWidthToCItems = 0;
		dojo.query("li", this.toc).forEach( function( item ){
			this.totalWidthToCItems += dojo.getMarginBox( item ).w; 
		}, this );			
		
		//dojo.style(this.toc, "width", this.totalWidthToCItems + "px");
		//55135: STG: Wizard usability/layout issues in OneUI
		//Bobby: 12Oct2012, ToC is wrapping and need additional 1px
		//Randy: 20Oct2012 OOPS! Should have added tocMarginLeft, instead of hardcoding a constant!
		// This is because the margin of the container (toc, the ordered list) must be accounted for.
		/*
		 * 55819: The root cause comes from Dojo. dojo.getMarginBox() returns wrong value of item's
		 * width in Chrome. Work it around by setting minWidth instead of width to avoid toc being
		 * too narrow.
		 */
		if(this.tocMarginLeft > 0) //Bobby:21Oct2012, Just in case it is -ve
			dojo.style(this.toc, "minWidth", (this.totalWidthToCItems + this.tocMarginLeft) + "px");
		else{
			dojo.style(this.toc, "minWidth", (this.totalWidthToCItems + 2) + "px"); //Bobby:21Oct2012, Typically 2
		}	
	},
	
	/**
	* Right slider button is made visible if not all step arrows can
	* be displayed in the pane.  Once user clicks on button and slides
	* the arrows, then the left slider button is added.  Both buttons
	* remain visible unless the pane is resized so that all steps can
	* be displayed.
	*/
	updateSliderUI: function (newWidth) {
		// newWidth will be passed in when toc content pane is being resized;
		// use it instead of scrollWidth since that value may not be updated yet
		var slideWidth = (newWidth != null) ? newWidth : dojo.style(this.tocSteps, "width");
		
		if (this.totalWidthToCItems >= slideWidth - this.tocStepsMargin) {
			if (this.ltr) {
			// Show right slider button only if no buttons currently showing
				if (this.sliderRight.style.display == "none") {
					dojo.style(this.sliderRight, "display", "inline");
					this.visibleButtonWidth = this.oneButtonWidth;        
				} else {
					this.finishSlide(this.sliderLeft, slideWidth);
				}
			} else {
				// Show left slider button only if no buttons currently showing
				if (this.sliderLeft.style.display == "none") {
					dojo.style(this.sliderLeft, "display", "inline");
					this.visibleButtonWidth = this.oneButtonWidth;        
				} else {
					this.finishSlide(this.sliderRight, slideWidth);
				}
			}
			//55564: NaN error causes wizard launch to fail in IE 8
			//Bobby: 17Oct2012, adding check to avoid setting NaN as width
			if (slideWidth >= this.visibleButtonWidth) {
				dojo.style(this.sliderOuterDiv, "width", (slideWidth - this.visibleButtonWidth) + "px");
				this.oldTocStepsWidth = slideWidth;
			}
		} else if (this.visibleButtonWidth > 0) {
			// no need for slider, hide everything
			dojo.style(this.sliderLeft, "display", "none");        
			dojo.style(this.sliderRight, "display", "none");        
			dojo.style(this.sliderOuterDiv, "width", "100%");
			dojo.style(this.toc, "left", "0px");
			this.pos = 0;
			this.slideDirection = "";
			this.visibleButtonWidth = 0;
		}    
	},

	/**
	* Handles case where both slider buttons are visible, toc has been 
	* scrolled so first and last step(s) are no longer visible and window 
	* is being resized.  As the window width is increased, once the last 
	* step is visible the toc needs to be slid so that the first step(s) 
	* become visible too.
	*/
	finishSlide: function (button, slideWidth) {
		var defaultToc = 0;
		if (button.style.display != "none") {
			var tocLeft = dojo.style(this.toc, "left");
			if (this.ltr) {
				// left-to-right direction
				if (tocLeft < 0) {
					var tocWidthShowing = dojo.style(this.toc, "width") + tocLeft;
					if (slideWidth > (tocWidthShowing + this.visibleButtonWidth + this.tocMarginLeft)) {
						this.pos = tocLeft + slideWidth - this.oldTocStepsWidth;
						dojo.style(this.toc, "left", this.pos + "px");
					}                  
				}
			} else {
				// right-to=left direction
				if (tocLeft > 0) {
					var tocWidthShowing1 = defaultToc - (dojo.style(this.toc, "width") - tocLeft);
					if (slideWidth > (tocWidthShowing1 + this.visibleButtonWidth + this.tocMarginLeft)) {
						this.pos = tocLeft - slideWidth + this.oldTocStepsWidth;
						dojo.style(this.toc, "left", this.pos + "px");
					}                            
				}
			}
		}
	},
  
	/**
	* Called to show the second slider button after first slider button
	* has been clicked.
	*/  
	showSecondButton: function (button, marginStyle, multiplier) {
		this.visibleButtonWidth *= 2;
		this.pos = multiplier * this.oneButtonWidth;
		dojo.style(button, "display", "inline");
		dojo.style(this.toc, "left", this.pos + "px");
		dojo.style(this.sliderOuterDiv, "width", (dojo.style(this.tocSteps, "width") - this.visibleButtonWidth) + "px");
		dojo.style(this.toc, marginStyle, this.tocMarginLeft + "px");    
	},
  
	/**
	* Returns the number of pixels that have been scrolled due to
	* tabbing.
	*/
	getScrollAmount: function () {
		var scrollAmt = 0;
		if (this.sliderOuterDiv.scrollLeft > 0) { // IE
			scrollAmt = this.sliderOuterDiv.scrollWidth - this.sliderOuterDiv.scrollLeft - this.sliderOuterDiv.offsetWidth;
		} else if (this.sliderOuterDiv.scrollLeft < 0) {  // FF
			scrollAmt = scrollAmt - this.sliderOuterDiv.scrollLeft;
		}
		return scrollAmt;
	},
  
	/**
	* Invoked when user clicks on right slider button
	*/
	slideRight: function () {
		if (this.slideDirection == "") {
			this.showSecondButton(this.sliderLeft, "marginLeft", -1);
		} else {
			this.slideStop();
		}
		this.slideDirection = "R";
		// determine number of pixels to slide adjusting for any scrolling
		// from tabbing if necessary
		var slideWidth = dojo.style(this.sliderOuterDiv, "width") - this.tocMarginLeft;    
		var leftToSlide;
		if (this.ltr) {
			leftToSlide = this.totalWidthToCItems - slideWidth + this.pos - this.sliderOuterDiv.scrollLeft;
		} else {
			leftToSlide = this.pos + this.getScrollAmount();
		}
		// don't slide too much
		if (leftToSlide > slideWidth) {
			leftToSlide = slideWidth;
		}
		// call recursive function to do actual slide
		if (leftToSlide > 0) {
			this._slideRight(leftToSlide - this.pos);
		}
	},
  
	_slideRight: function (totalToSlide) {    
		if (this.pos == 0) {
			this.pos = -1;
		}
		this.pos -= this.slidePixel;
		if (this.pos < -totalToSlide) {
			var diff = -this.pos - totalToSlide;
			this.pos += this.slidePixel;
			if (diff < this.slidePixel) {
				this.pos -= diff;  // any last pixels if not multiple of slidePixel number
				dojo.style(this.toc, "left", this.pos + "px");
			}
			return;
		}
		dojo.style(this.toc, "left", this.pos + "px");
		this.t = setTimeout(dojo.hitch(this, this._slideRight, totalToSlide), this.slideDelay);    
	},
  
	/**
	* Invoked when user clicks on left slider button
	*/
	slideLeft: function () {    
		if (this.slideDirection == "") {
			this.showSecondButton(this.sliderRight, "marginRight", 1);
		} else {
			this.slideStop();
		}
		this.slideDirection = "L";
			
		// determine number of pixels to slide adjusting for any scrolling
		// from tabbing
		var slideWidth = dojo.style(this.sliderOuterDiv, "width") - this.tocMarginLeft;    
		var leftToSlide;
		if (this.ltr) {
			leftToSlide = -this.pos + this.sliderOuterDiv.scrollLeft;
		} else {
			leftToSlide = this.totalWidthToCItems - slideWidth - this.pos - this.getScrollAmount();
		}
		// don't slide too much
		if (leftToSlide > slideWidth) {
			leftToSlide = slideWidth;
		}
		// call recursive function to do actual slide
		if (leftToSlide > 0) {
			this._slideLeft(leftToSlide);
		}
	},
  
	_slideLeft: function (totalToSlide) {
		if (this.pos == 0) {
			this.pos = 1;
		}
		this.pos += this.slidePixel;
		totalToSlide -= this.slidePixel;
		if (totalToSlide < 0) {
			this.pos -= this.slidePixel;
			if (-totalToSlide < this.slidePixel) {
				this.pos += -totalToSlide;  // any last pixels if not multiple of slidePixel number
				dojo.style(this.toc, "left", this.pos + "px");
			}
			return;
		}
		dojo.style(this.toc, "left", this.pos + "px");
		this.t = setTimeout(dojo.hitch(this, this._slideLeft, totalToSlide), this.slideDelay);    
	},

	/**
	* Cancels the set timer so sliding is stopped
	*/
	slideStop: function () {
		clearTimeout(this.t);
	},
  
	/**
	* Resets slider so first arrow is visible.
	*/
	reset: function () {
		this.pos = 0;
		dojo.style(this.toc, "left", "0px");
		if (this.ltr) {
			this.sliderOuterDiv.scrollLeft = 0;
		}
	}
});

});
