/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/activealerts/ActiveAlertsIgnoretDialogTask", [
    "dijit/_WidgetBase",
    "dijit/form/Button",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/on",
    "dojo/topic",
    "dojo/dom-construct",
    "dojo/dom-style",
    "dijit/registry",
    "dijit/layout/ContentPane",
    "dojo/i18n!ibm/usmi/nls/frameworkResources",
    "dojo/i18n!ibm/usmi/nls/defaultTaskResources",
    "dojo/i18n!ibm/usmi/activealerts/nls/ActiveAlertsResources",
    "ibm/usmi/core/utils/Logger",
    "ibm/tivoli/navmodel/Selection",
    "dijit/form/RadioButton",
    "dojo/string",
    "dojo/dom-attr",
    "ibm/usmi/util/DefaultMessageHandler"
],

function(_WidgetBase, Button, declare, lang, on, topic, construct, style, registry, ContentPane, FrameworkMRI,
        DefaultMRI, ActiveAlertsMRI, Logger, Selection, RadioButton, string, domAttr, DefaultMessageHandler) {
    return declare("ibm.usmi.activealerts.ActiveAlertsIgnoreDialogTask", [
        _WidgetBase,
        ContentPane
    ], {

        // summary:
        //		This is the task for ignoring the status entry or status set . To ignore the individual status entry,
        //      just select the "Selected resource only", to ignore the entire status set, then select "All resource" option
        // description:
        //		Coming soon

        // private members       
        _args: null,
        _task: null,
        _selectionName: null,
        FrameworkResources: FrameworkMRI,
        DefaultResources: DefaultMRI,
        ActiveAlertsResources: ActiveAlertsMRI,
        _ActiveAlertsLogger: null,

        constructor: function(/*constructor_kwArgs*/kwArgs) {

            this._args = {
                launchParms: "",
                menuItem: "",
                context: ""
            };

            lang.mixin(this._args, kwArgs);

            // create the logger
            this._ActiveAlertsLogger = new Logger("ibm.usmi.activealerts", "ActiveAlerts");

            // Run any parent postCreate processes - can be done at any point
            this.inherited(arguments);
            if (this._ActiveAlertsLogger.isFinest()) {
                this._ActiveAlertsLogger.exiting("Active Alerts Ignore Dialog Task is constructed", "constructor");
            }

        },

        startup: function() {
            this.inherited(arguments);
        },

        postCreate: function() {

            var getTasksError = function(error, request) {
                var msgHandler = new DefaultMessageHandler({});
                msgHandler.displayException(error);
            };

            this.getDataSet().getTasks({
                onComplete: lang.hitch(this, this._getTasksComplete),
                onError: getTasksError,
                taskCategory: "menu", //TODO: Could I specifically request nav hints instead of all tasks?
                selection: this.getSelection()
            });

            // Get a DOM node reference for the root of our widget
            var domNode = this.domNode;

            // Run any parent postCreate processes - can be done at any point
            this.inherited(arguments);
            var selections = this._args.context.selection.getItems();
            var selectionName = selections[0].getLabel();
            this._selectionName = selectionName;
            if (!(this._args.launchParms.confirmationText)) {
                this._args.launchParms.confirmationText = string.substitute(this.DefaultResources.CONFIRMMSG, [
                    1,
                    this._selectionName
                ]);
            }

            // text div		
            var activeAlertDiv = construct.create("div", {
                "innerHTML": this._args.launchParms.confirmationText,
                style: "padding: 5px"
            });

            construct.place(activeAlertDiv, domNode);

            var radioOnlyDiv = construct.create("div");
            style.set(radioOnlyDiv, "padding", "5px 5px 5px 20px");
            construct.place(radioOnlyDiv, domNode);

            var radioOnly = new RadioButton({
                value: "only",
                id: this.id + "_radioOnly",
                checked: true,
                name: "resource"
            }, radioOnlyDiv.appendChild(construct.create("div")));

            // label for radioOnly
            var labelVal = construct.create('label');
            domAttr.set(labelVal, "for", "radioOnly");
            domAttr.set(labelVal, "innerHTML", this.ActiveAlertsResources.resourceOnly);
            radioOnlyDiv.appendChild(labelVal);

            var radioaAllDiv = construct.create("div");
            style.set(radioaAllDiv, "padding", "5px 5px 5px 20px");
            construct.place(radioaAllDiv, domNode);

            var radioaAll = new RadioButton({
                id: this.id + "_radioAll",
                value: "all",
                name: "resource"
            }, radioaAllDiv.appendChild(construct.create("div")));

            // label for radioAll
            var labelVal2 = construct.create('label');
            domAttr.set(labelVal2, "for", "radioAll");
            domAttr.set(labelVal2, "innerHTML", this.ActiveAlertsResources.allResources);
            radioaAllDiv.appendChild(labelVal2);

            var activeAlertBottomDiv = construct.create("div");
            style.set(activeAlertBottomDiv, "padding", "5px 5px 5px 10px");
            construct.place(activeAlertBottomDiv, domNode, "last");

            var saveButton = new Button({
                id: this.id + "_DialogOKButton",
                label: this.FrameworkResources.OK
            }, activeAlertBottomDiv.appendChild(construct.create("div")));

            var cancelButton = new Button({
                id: this.id + "_DialogCancelButton",
                label: this.FrameworkResources.CANCEL
            }, activeAlertBottomDiv.appendChild(construct.create("div")));

            on(saveButton, "click", lang.hitch(this, "onOK"));
            on(cancelButton, "click", lang.hitch(this, "onCancel"));

        },

        _getTasksComplete: function(tasks) {
            // summary:
            //      get task from id
            var i, subTasks;
            //Want to loop through tasks looking for
            if (tasks) {
                for (i = 0; i < tasks.length; i++) {
                    var task = tasks[i];
                    // found are task.
                    if (task.getId() === this.getTaskId()) {

                        this.setTask(task);

                        break;
                    } else if (task.getChildTasks()) {
                        subTasks = task.getChildTasks();
                        if (subTasks.length > 0) {
                            this._getTasksComplete(subTasks);
                            if (this.getTask()) {
                                break;
                            }
                        }
                    }
                }
            }
        },

        onCancel: function() {
            // summary:
            //		cancel this task
            topic.publish("DefaultDialogDismissal", []);
            registry.byId(this.id + "_DialogCancelButton").destroyRecursive();
            registry.byId(this.id + "_radioOnly").destroyRecursive();
            registry.byId(this.id + "_radioAll").destroyRecursive();
            this.destroyRecursive();
            this.inherited(arguments);
        },

        onOK: function() {
            // summary:
            //		execute the task
            var taskParms = {
                resourceOnly: registry.byId(this.id + "_radioOnly").get("checked")
            };
            this.getTask().execute({
                onError: lang.hitch(this, this._handleExecuteOnServerError),
                selection: this.getSelection(),
                selectionParms: taskParms,
                sync: true
            });

            this.onCancel();
        },

        getDataSet: function() {
            // summary:
            //      returns the dataset
            return this._args.context.dataset;
        },

        getTaskId: function() {
            // summary:
            //      returns the task id
            return this._args.menuItem.actionID;
        },

        getTask: function() {
            // summary:
            //      returns the task
            return this._task;
        },

        setTask: function(t) {
            // summary:
            //      set the task
            // t:
            //      the task
            this._task = t;
        },

        getSelection: function() {
            // summary:
            //      returns the selection
            var selectedThings = this._args.context.selection;
            var currentSelection = null;
            if (selectedThings) {
                currentSelection = new Selection({
                    version: this.getDataSet().getVersion()
                });
                currentSelection.addItems(selectedThings.getItemIds());
                currentSelection.addLinks(selectedThings.getLinkIds());
                currentSelection.addRelationships(selectedThings.getRelationshipIds());
            }
            return currentSelection;
        },

        getBodyHeight: function() {
            // summary:
            //      returns the body height
            return (0.1 * ((window.innerHeight !== undefined) ? window.innerHeight : document.body.offsetHeight));
        },

        _handleExecuteOnServerError: function(error, exception, ioArgs) {
            // summary:
            //		handle the error on server side

            if (this._ActiveAlertsLogger.isError()) {
                this._ActiveAlertsLogger.error(">>> error executing server task = " + error,
                        "_handleExecuteOnServerError");
            }

            var msgHandler = new DefaultMessageHandler({});
            msgHandler.displayException(error);
        }
    });
});
