/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
 
var iscWindow = undefined;	// Store opened ISC window

// Maintain a flag to tell whether the NGUI window has focus
var nguiFocus = false;
window.onfocus = function() {nguiFocus=true;};
window.onblur = function() {nguiFocus=false;};

var warningShowed = false;	// A flag to control we only show the warning once.
 
/*
	forces login when launch in context called from NGUI, this is so ISC doesn't ask for user credentials
	before opening the page in ISC
*/
function forceLogin (xss, url, host) {
	var loginForm = document.createElement("form");
	loginForm.setAttribute("method", "POST");
	loginForm.setAttribute("action", "/ibm/console/secure/securelogon.do?XSS=" + xss);
	loginForm.setAttribute("name", "loginConflictForm");
	loginForm.setAttribute("style", "display: none");

	var field1 = document.createElement("input");
	field1.setAttribute("type", "radio");
	field1.setAttribute("name", "action");
	field1.setAttribute("value", "force");
	field1.setAttribute("checked", "checked");

	var field2 = document.createElement("input");
	field2.setAttribute("type", "radio");
	field2.setAttribute("name", "action");
	field2.setAttribute("value", "continue");

	loginForm.appendChild(field1);
	loginForm.appendChild(field2);

	var xhrArgs = {
		form : loginForm,
		handleAs : "text",
		load : function (data) {
			openISCWindow(host, url);
		},
		error : function (error) {
		}
	};
	dojo.xhrPost(xhrArgs);

}

/* Opens an ISC window using a specific window name. The window name used is the ISC_ plus the session ID
   retrieved from the server.

	If the window cannot be focused (because it is a tab behind other tabs), then shows an error message.
*/
function openISCWindow (host, url) {

	// IE won't happy with a window title with dash in it.
	var title = "ISC_" + sessionId;
	title = title.replace(/-/g, "_");

	var windowFeatures = "resizable=1,scrollbars=1";
	if(dojo.isFF) {
		// size/position FF manually so that if it is maximized, it won't just overlay NGUI
		windowFeatures += ",top=" + (window.screenY+22);
		windowFeatures += ",left=" + (window.screenX+22);
	}
	iscWindow = window.open(url, title, windowFeatures);

	// Test for popup blockers.
	if(iscWindow === null || typeof(iscWindow) == 'undefined')
	{		
		showAlert(popupBlockerWarningMessage);
	}
	
	iscWindow.focus();
	
	// set a timeout for IE
	setTimeout( function() {
		if(nguiFocus && !warningShowed) {
			// If the current window didn't lose focus, which means iscWindow is not focused,
			// and we didn't show this message before, show this message.
			require(["dojo/_base/lang", "dojo/i18n!ibm/usmi/nls/frameworkResources"], function(djLang, MRI) {
				var msg1 = djLang.replace(MRI["LaunchedInTabs1"], [iscWindow.document.title]);
				var msg2 = djLang.replace(MRI["LaunchedInTabs2"], [iscWindow.document.title]);
				showWarning(msg1, msg2, false);
				warningShowed = true;
			});
		}
	}, 500 );
}

/*
	Show a alert dialog with messages
*/
function showAlert(msg) {
	require(["dojoe/messagedialog/MessageDialog"], function(Dialog){
		var msgDlg = new Dialog({
			message: msg
		});
		msgDlg.show();		
	});
}

/*
	Show some warning message in the main workarea with a title and details collapsed or not.
*/
function showWarning(title, description, isCollapsed) {
	require(["ibm/usmi/util/DefaultMessageHandler"], function(Handler){
		var handler = new Handler({});
		handler.displayMessage("", Handler.WARNING, title, description, isCollapsed);
	});	
}

/*
	checks to see if a user is already logged in to see if there should be a forced login or if
	the ISC window can just be opened
*/
function checkLogin(host, url) {
	var xhrArgs = {
		url : url,
		handleAs : "text",
		preventCache : true,
		load : function (data) {
			var str = new String(data);
			var start = str.indexOf("loginConflictForm");
			if (start > 0) {
				// another user logged in already
				// get XSS for post
				var begin = data.indexOf("XSS=", start) + 4;
				dojoConfig.iscSession = data.substring(begin, data.indexOf("\">", begin));
				forceLogin(dojoConfig.iscSession, url, host);
			} else {
				// open isc window
				var begin = data.indexOf('XSS="') + 5;
				dojoConfig.iscSession = data.substring(begin, data.indexOf("\"", begin));
				openISCWindow(host, url);
			}
		},
		error : function (error) {			
			// open it anyway
			openISCWindow(host, url);
		}
	};
	dojo.xhrGet(xhrArgs);
}

//launches a page using ISC's launch in context
//pageID: the ISC page ID for the page to launch
//targets: array of sourceTokens for this task, if any
//externalUrl: directly use externalUrl instead
function launchInContext (pageID, targets, externalUrl) {

	// Prepare url
	var timestamp = new Date().getTime();
	var url = window.location.protocol + '//' + location.host + '/ibm/console/xLaunch.do?pageID=' + pageID
			+ '&timestamp='+timestamp;
	if (targets) {
		for (var i = 0; i < targets.length; i++) {
			if (i === 0) {
				url = url + '&sourceToken=' + targets[0];
			} else {
				url = url + ',' + targets[i];
			}
		}
	}
	if (externalUrl) {
		//use externalUrl directly
		url = externalUrl;
	}

	//189718-XB133609 launch window in setTimeout, because some widgets need to do a little processing (like cleanup/close widget) which prevents 
	//                the ISC window from keeping focus. Adding a bit of delay ensures widget cleans up before launch, and ISC window retains focus
	var locationHost = location.host;
	setTimeout(function () {
		openISCWindow(locationHost, url);
		locationHost = null;
		url = null;
	}, 150);
}

/*
	This function is not used any more, but leave here in case
	we want to close all tab pages in the future.
	It demo. the hack to mimic pages of close all from ISC.
*/
function openISCWindowAfterCloseAll(url, title) {
	var XSS = localStorage.getItem("usmiLocalStorageKey");
	var manageOpenPagesUrl = "/ibm/console/navigation.do?XSS="+XSS;
		manageOpenPagesUrl += "&pageID=com.ibm.isclite.portlet.manageopenpages.page&moduleRef=com.ibm.isclite.ISCAdminPortlets&ISC.TASKTYPE=2&XSS=";
	
	// Close all pages if open manage open pages
	var getComplete = function(data) {
		var actionUrl = "/ibm/console/pa.do?_pa=1483046106&XSS="+XSS+"&.pa=true";
		var closeAllForm = document.createElement("form");
		closeAllForm.setAttribute("method", "POST");
		closeAllForm.setAttribute("action", actionUrl);

		var field1 = document.createElement("input");
		field1.setAttribute("type", "hidden");
		field1.setAttribute("name", "iscAdminPortletsRemoveRows");
		field1.setAttribute("value", "Close All Pages");

		var field2 = document.createElement("input");
		field2.setAttribute("type", "hidden");
		field2.setAttribute("name", "pagingCurrPageNum");
		field2.setAttribute("value", "1");
		closeAllForm.appendChild(field1);
		closeAllForm.appendChild(field2);
		
		var xhrPostArgs = {
			form : closeAllForm,
			handleAs : "text",
			load : function (data) {
				//openISCWindowActually(url, title);
			},
			error : function (error) {
				//openISCWindowActually(url, title);
			}
		};
		dojo.xhrPost(xhrPostArgs);	
	};
	
	var xhrGetArgs = {
		url : manageOpenPagesUrl,
		handleAs : "text",
		preventCache : true,
		load : dojo.hitch(this, getComplete),
		error : function (error) {
			//openISCWindowActually(url, title);
		}
	};
	dojo.xhrGet(xhrGetArgs);
}

function logoutFSM () {
	var logoutForm = document.createElement("form");
	logoutForm.setAttribute("method", "POST");
	logoutForm.setAttribute("action", "ibm_security_logout");
	logoutForm.setAttribute("name", "logout");
	logoutForm.setAttribute("style", "display: none");

	var field1 = document.createElement("input");
	field1.setAttribute("type", "submit");
	field1.setAttribute("name", "logout");
	field1.setAttribute("value", "LOGOUT");

	var field2 = document.createElement("input");
	field2.setAttribute("type", "hidden");
	field2.setAttribute("name", "logoutExitPage");
	var redirUrl = new Date().getTime();	// IE won't be happy without this.
	redirUrl = window.location.pathname + "?"+ redirUrl;
	field2.setAttribute("value", "");
	logoutForm.appendChild(field1);
	logoutForm.appendChild(field2);
	
	var xhrArgs = {
		form : logoutForm,
		handleAs : "text",
		load : function (data) {
			localStorage.clear();
			require([ "dojo/topic"
			], function (topic) {
				topic.publish("logout-event", null);
			});			
			window.location.replace(redirUrl);
		},
		error : function (error) {
			localStorage.clear();
			window.location.replace(redirUrl);
		}
	};
	dojo.xhrPost(xhrArgs);
}

function logoutISC () {
	var sessionID = localStorage.getItem("usmiLocalStorageKey");
	if (sessionID) {
		var xhrArgs = {
			url : "/ibm/console/logout.do?XSS=" + sessionID,
			handleAs : "text",
			preventCache : true,
			load : function (data) {
				logoutFSM();
			},
			error : function (error) {
				logoutFSM();
				require([ "dojo/topic"
				], function (topic) {
					topic.publish("logout-event", null);
				});				
			}
		};
		dojo.xhrGet(xhrArgs);
	}
}