/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/dashboard/DatasetItemStoreAdapter", ["dojo/_base/declare",
		"ibm/tivoli/navmodel/Dataset",
		"ibm/tivoli/navmodel/DatasetItemsStore",
		"ibm/usmi/util/DefaultMessageHandler",
		"dojo/_base/Deferred",
		"dojo/_base/lang"
	], function (declare, Dataset, DatasetItemsStore, DefaultMessageHandler, Deferred, lang) {
	return declare(null, {
		dataset: "EVENT_LOG",
		sourceToken: null,
		_sort: [{attribute:"DateTime", descending: true}],
		_start: 0,
		_count: 3,
		_defaultProperties: ['Source', 'Severity', 'EventText', 'DateTime'],
		new_dataset_store: null,
		
		_deferredDataStore: null,
		
		constructor: function(/*constructor_kwArgs*/ kwArgs) {	
			declare.safeMixin(this, kwArgs);	
		},
		
		
		/**
		*/
		getStore: function(){
			this._deferredDataStore = new Deferred();
			var options={
				onComplete : lang.hitch(this, function (new_dataset, kwArgs) {
					this.new_dataset_store = new DatasetItemsStore({
                        dataset: new_dataset,
						defaultProperties: this._defaultProperties
                    });
					this.fetchItems();
				}),
				onError : function (error, exception, ioArgs) {
					var msgHandler = new DefaultMessageHandler({});
					msgHandler.displayException(exception);
				},
				baseUrl : "/ibm/tivoli/rest",
				provider : "USMI",
				datasource : "",
				dataset : this.dataset,
				clearCacheOnRefresh : true,
				createEventRouter: false, //do not create event router
				defaultProperties : []
			};
			if(this.sourceToken){
				//console.debug("sourceToken:"+this.sourceToken);
				options.parameters = {sourceToken: this.sourceToken};
			}
			var tctDataset = new Dataset(options);
			return this._deferredDataStore;
		},
		
		fetchItems: function(){
			if(!this.new_dataset_store){
				//console.error("data store is not available yet!");
				return;
			}
			this.new_dataset_store.fetch({
				onComplete:  function (items, kwArgs) {
					//console.debug("finishing load items");
					this._deferredDataStore.resolve(items);
				},
				onError : function (error, exception, ioArgs) {
					var msgHandler = new DefaultMessageHandler({});
					msgHandler.displayException(exception);
				},
				start: this._start,
				count: this._count,
				sort: this._sort,
				scope: this
			});
		}
		
		
		
    });
});
