/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/tasks/DefaultConfirmationDialogTask", [
    "dijit",
    "dojo",
    "dojox",
    "dijit/_WidgetBase",
    "dijit/_Templated",
    "dijit/Dialog",
    "dijit/form/Button",
    "dojox/data/dom",
    "ibm/tivoli/navmodel/Trace",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/on",
    "dojo/topic",
    "dojo/dom-construct",
    "dojo/dom-style",
    "dojo/_base/window",
    "dijit/registry",
    "dijit/layout/ContentPane",
    "dojo/i18n!ibm/usmi/nls/frameworkResources",
    "ibm/usmi/util/DefaultMessageHandler",
    "ibm/usmi/core/utils/Logger",
    "dojo/i18n!ibm/usmi/nls/defaultTaskResources"
],

function (dijit, dojo, dojox, _WidgetBase, _Templated, _Dialog, _Button, dom, theTrace, declare, lang, on, topic,
        construct, style, win, registry, ContentPane, FrameworkMRI, DefaultMessageHandler, Logger, DefaultMRI) {

	//Create our widget!
	return declare("ibm.usmi.tasks.DefaultConfirmationDialogTask", [
	    _WidgetBase,
	    ContentPane
	], {

	    // private members       
	    _args : null,
	    _task : null,
	    _selectionName : null,
	    FrameworkResources : FrameworkMRI,
	    DefaultResources : DefaultMRI,
		// Logger
	    DefaultConfirmationLogger : new Logger("ibm.usmi.tasks.DefaultConfirmationDialogTask", "DefaultConfirmationDialogTask"),


	    constructor : function (/*constructor_kwArgs*/kwArgs) {
		    // summary:
		    //		Coming soon
		    // description:
		    //		Coming soon

		    this._args = {
		        launchParms : "",
		        menuItem : "",
		        context : ""
		    };

		    lang.mixin(this._args, kwArgs);

		    //	this._datasetId = kwArgs.datasetId;
		    //	this._taskId = kwArgs.taskId;
		    //	this._oids = kwArgs.oIds;

		    // Run any parent postCreate processes - can be done at any point
		    this.inherited(arguments);
			if (this.DefaultConfirmationLogger.isFine()) {
				this.DefaultConfirmationLogger.fine("DefaultConfirmationDialogTask: Constructor");
			}
	    },

	    startup : function () {
		    this.inherited(arguments);
	    },

	    postCreate : function () {

		    var getTasksError = function (error, request) {
			    this._navWidget.hideBusyPane();

			    //Give the user some feedback
			    this.displayException(error);
		    };

		    this.getDataSet().getTasks({
		        onComplete : lang.hitch(this, this._getTasksComplete),
		        onError : lang.hitch(this, this.getTasksError),
		        taskCategory : "menu", //TODO: Could I specifically request nav hints instead of all tasks?
		        selection : this.getSelection()
		    });

		    // Get a DOM node reference for the root of our widget
		    var domNode = this.domNode;

		    // Run any parent postCreate processes - can be done at any point
		    this.inherited(arguments);
		    var selections = this._args.context.selection.getItems();
		    var selectionName = selections[0].getLabel();
		    this._selectionName = selectionName;
		    if (!(this._args.launchParms.confirmationText)) {
			    this._args.launchParms.confirmationText = dojo.string.substitute(this.DefaultResources.CONFIRMMSG, [
			        1,
			        this._selectionName
			    ]);
		    }

		    // text div		
		    var dialogContentDiv = construct.create("div", {
		        "innerHTML" : this._args.launchParms.confirmationText,
		        style : "padding: 5px"
		    });

		    construct.place(dialogContentDiv, domNode);

		    var bottomButtonsDiv = construct.create("div");
		    style.set(bottomButtonsDiv, "padding", "5px 5px 5px 10px");
		    construct.place(bottomButtonsDiv, domNode, "last");

		    var saveButton = new dijit.form.Button({
		        id : this.id + "_DialogOKButton",
		        label : this.FrameworkResources.OK,
		    }, bottomButtonsDiv.appendChild(construct.create("div")));

		    var cancelButton = new dijit.form.Button({
		        id : this.id + "_DialogCancelButton",
		        label : this.FrameworkResources.CANCEL,
		    }, bottomButtonsDiv.appendChild(construct.create("div")));

		    on(saveButton, "click", lang.hitch(this, "onOK"));
		    on(cancelButton, "click", lang.hitch(this, "onCancel"));
	    },

	    // get task from id
	    _getTasksComplete : function (tasks) {

		    //Want to loop through tasks looking for
		    if (tasks) {
			    for ( var i = 0; i < tasks.length; i++) {
				    var task = tasks[i];
				    // found are task.
				    if (task.getId() === this.getTaskId()) {

					    this.setTask(task);

					    break;
				    } else if (task.getChildTasks()) {
					    var subTasks = task.getChildTasks();
					    if (subTasks.length > 0) {
						    this._getTasksComplete(subTasks);
						    if (this.getTask()) {
							    break;
						    }
					    }
				    }
			    }
		    }
	    },

	    //override the onCancel method to also destroy the whole widget.
	    onCancel : function () {
		    // summary:
		    //		Coming soon
		    // description:
		    //		Coming soon
		    setTimeout('dijit.byId(\"' + this.id + '\").destroyRecursive();', 200);

		    // the above line was not deleting the button id and so additional step
		    var buttonId = this.id + "_DialogCancelButton";
		    if (buttonId) {
			    dijit.byId(this.id + "_DialogCancelButton").destroyRecursive();
		    }

		    topic.publish("DefaultDialogDismissal", []);
		    this.inherited(arguments);
	    },

	    //override the onCancel method to also destroy the whole widget.
	    onOK : function () {
		    // summary:
		    //		Coming soon
		    // description:
		    //		Coming soon

		    this.getTask().execute({
		        onError : dojo.hitch(this, this._handleExecuteOnServerError),
		        onComplete : lang.hitch(this, this._getExecuteComplete),
		        selection : this.getSelection()
		    });

		    setTimeout('dijit.byId(\"' + this.id + '\").destroyRecursive();', 200);

		    // the above line was not deleting the button id and so additional step
		    var buttonId = this.id + "_DialogOKButton";
		    if (buttonId) {
			    dijit.byId(this.id + "_DialogOKButton").destroyRecursive();
		    }

		    topic.publish("DefaultDialogDismissal", []);
		    this.inherited(arguments);
	    },
	    
	    _getExecuteComplete : function (
			result, arg) {
			var r = result;
			var a = arg;
			var moduleName = "_getExecuteComplete";
			// TODO:  Consider how to add a link to the job details in the message -- based on the job ID....r.JobId
			var theMessageTitle = "";
			var msgDesc = "" ;		
	
			// Display the message...
			var msgHandler = new DefaultMessageHandler();
			var msgId = ""; // not specifying a message ID
			var msgSeverity = DefaultMessageHandler.SUCCESS;
            // Server side has not been implemented yet
			//msgHandler.displayMessage(msgId, msgSeverity, theMessageTitle, msgDesc, true);
		},

	    getDataSet : function () {
		    return this._args.context.dataset;
	    },

	    getTaskId : function () {
		    return this._args.menuItem.actionID;
	    },

	    getTask : function () {
		    return this._task;
	    },

	    setTask : function (t) {
		    this._task = t;
	    },

	    getSelection : function () {
		    var selectedThings = this._args.context.selection;
		    var currentSelection = null;
		    if (selectedThings) {
			    currentSelection = new ibm.tivoli.navmodel.Selection({
				    version : this.getDataSet().getVersion()
			    });
			    currentSelection.addItems(selectedThings.getItemIds());
			    currentSelection.addLinks(selectedThings.getLinkIds());
			    currentSelection.addRelationships(selectedThings.getRelationshipIds());
		    }
		    return currentSelection;
	    },

	    getBodyHeight : function () {
		    return (0.1 * ((window.innerHeight !== undefined) ? window.innerHeight : document.body.offsetHeight));
	    },

	    _handleExecuteOnServerError : function (error, exception, ioArgs) {
	    	var err = error;
			var exc = exception;
			var a = ioArgs;
			var moduleName = "_handleExecuteOnServerError";
		    var theMessageTitle = this._selectionName;
			var theMessageDesription = exc.getMsgText();
			var msgDesc = theMessageDesription;		
			
			if (this.DefaultConfirmationLogger.error()) {
				this.DefaultConfirmationLogger.error("DefaultConfirmationDialogTask:  Failed to rename system: " +
					this._selectionName, moduleName);
				this.DefaultConfirmationLogger
					.error("DefaultConfirmationDialogTask:  The error received is:  " + err, moduleName);
				this.DefaultConfirmationLogger.error("DefaultConfirmationDialogTask:  The exception is:  " + exc, moduleName);
			}
		

			// Display the message...
			var msgHandler = new DefaultMessageHandler();
			var msgId = ""; // not specifying a message ID
			var msgSeverity = DefaultMessageHandler.ERROR;
			msgHandler.displayMessage(msgId, msgSeverity, theMessageDesription, msgDesc, true);
	    },
	}); // and that's it!
});
