/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

define("ibm/usmi/util/BaseWidget", [
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/hash",
	"ibm/usmi/widgets/util/Constants"
],

function (dojo, declare, lang, hash, Constants) {

	return declare([], {

		// private members
		// object to hold initial arguments, pass in via constructor, html tag, or url
		_initArgs : null,

		// common arguments
		providerID : "",
		collectionID : "",
		sourceToken : "",

		subscription : null,

		// update target method, subclasses override
		updateTargetMethod : "_updateTargets",

		constructor : function (/*constructor_kwArgs*/kwArgs) {

			this._initArgs = {
				sourceToken : null,
				collectionID : null,
				providerID : null
			};

			// mixin parameters that were passed in on the constructor, html tag attributes...
			dojo.mixin(this._initArgs, kwArgs);

			// mixin parameters that were specified on the URL, URL parms override all.
			dojo.mixin(this._initArgs, this.getUrlParmsObj());

			// mixin hash parameters
			dojo.mixin(this._initArgs, this.getUrlHashObj());

			// set provider, collection, source token passed in via constructor, html attribute, url
			this.providerID = this.getInitArg(Constants.PROVIDER);
			this.collectionID = this.getInitArg(Constants.COLLECTION);
			this.sourceToken = this.getInitArg(Constants.SOURCE_TOKEN);

			// Register for a re-target event
			this.subscription = dojo.subscribe(Constants.TOPIC_RETARGET, this, this.updateTargetMethod);
		},

		getInitArg : function (/*String*/key) {
			// summary:
			//        Returns a specific initial argument passed to the widget
			return this._initArgs[key];
		},

		getUrlParmsObj : function () {
			return dojo.queryToObject(dojo.doc.location.search.substr((dojo.doc.location.search[0] === "?" ? 1 : 0)));
		},

		getUrlParm : function (/*String*/key) {
			var urlObj = this.getUrlParmsObj();
			return urlObj[key];

		},

		getUrlHashObj : function () {
			return dojo.queryToObject(hash());
		},

		getUrlHashParm : function (/*String*/key) {
			var hashObj = this.getUrlHashObj();
			return hashObj[key];
		},

		setUrlHashObj : function (hashObj) {
			dojo.hash(dojo.objectToQuery(hashObj));
		},

		setUrlHash : function (/*String*/hashString, /*boolean*/replace) {
			dojo.hash(hashString, replace);
		},

		updateUrlHash : function (/*String*/hashkey, /*String*/hashValue) {
			var obj = this.getUrlHashObj(); // get
			obj[hashkey] = hashValue;
			return dojo.objectToQuery(obj); // set
		},

		//accept an object, take all the key/values and append/replace to current hash, return the string
		//doesn't actually call dojo.hash, just gives back the updated string
		updateHashValues : function (/*Object*/hashObject) {
			var key, obj = this.getUrlHashObj();
			for (key in hashObject) {
				obj[key] = hashObject[key];
			}
			return dojo.objectToQuery(obj);
		},

		//accept an object, take all the key/values and append/replace to current hash, return the string
		//doesn't actually call dojo.hash, just gives back the updated string
		updateHashObjectValues : function (/*Object*/hashObject) {
			var key, obj = this.getUrlHashObj();
			for (key in hashObject) {
				obj[key] = hashObject[key];
			}
			return obj;
		},

		// This is the same function provided by dojo.hitch, with an added check to make
		// sure the class object hasn't been destroyed.   By calling this method instead of 
		// dojo.hitch directly helps cover the scenario when back (which destroys the widget) while
		// callbacks were being processed.
		alive : function (scope, method) {
			if (arguments.length > 2) {
				return lang._hitchArgs.apply(dojo, arguments); // Function
			}
			if (!method) {
				method = scope;
				scope = null;
			}
			if (lang.isString(method)) {
				scope = scope || dojo.global;
				if (!scope[method]) {
					throw ([
						'alive: scope["',
						method,
						'"] is null (scope="',
						scope,
						'")'
					].join(''));
				}
				return function () {
					// on call if destroy has not been called
					if (!scope.isDestroyed()) {
						return scope[method].apply(scope, arguments || []);
					}
				}; // Function
			}
			return !scope ? method : function () {
				// on call if destroy has not been called
				if (!scope.isDestroyed()) {
					return method.apply(scope, arguments || []);
				}
			}; // Function
		},

		isDestroyed : function () {
			// subclass can override method to indicate whether or not their destroy method has been called
			// alive method will check if isDestroyed is false before invoking callback method
			return false;
		},

		_updateTargets : function (/*ibm.tivoli.navmodel.Dataset*/dataset, /*ibm.tivoli.navmodel.Item[]*/items) {
			console.log("Base Widget _updateTargets called, no subclass is listening");
		},
		destroy : function () {
			if (this.subscription) {
				this.subscription.remove();
				this.subscription = null;
			}
			this.inherited(arguments);
		}
	});
});
