/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/util/DataStore", [
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/array",
	"ibm/tivoli/tip/navwidget/dijit/DefaultMessageHandler",
	"ibm/tivoli/navmodel/DatasetItemsStore",
	"ibm/tivoli/navmodel/Dataset"
], function (declare, djLang, djArray, DefaultMessageHandler, DatasetItemsStore, Dataset) {
	return declare([], {
		_dataset : null,
		_items : null,
		_tasks : null,
		_datasetJson : null,
		_eventCountToNotify : 1, //default is 1, since we always get the dataset event
		_currentEventCount : 0,
		constructorParms : null,
		datasetArgs : null,
		/* constructorParms:
		 * You can construct the dataset in 2 ways. 
		 * 1. Use it to retrieve a regular collection, passing in getItems, getTasks, and retrieveItemProperties. 
		 * 2. Use it to retrieve a NGen task using a JSON object, passing in useJson
		 * getItems	- Boolean - retrieve collection items true/false - Default: true if useJson == false
		 * getTasks - Boolean - retrieve collection tasks true/false - Default: true if useJson == false
		 * retrieveItemProperties - Boolean - Retrieve collection items properties true/false - Default: true if useJson == false
		 * useJson - don't retrieve items, instead, get json string from dataset
		 * parameters - parameters to pass in to the collection creations, such as homeParms or source token 
		 * for ex:
		 * parameters: {homeParms : 'filterThis', sourceToken : 3445}
		 * initReadyCallback(Required) - method reference to call back when data retrieval is complete    
		*/

		constructor : function (args, additionalDatasetArgs) {
			this.constructorParms = {};
			this.datasetArgs = {};
			djLang.mixin(this.constructorParms, args);
			if (!this.constructorParms.parameters) {
				this.constructorParms.parameters = {};
			}

			if (additionalDatasetArgs) {
				djLang.mixin(this.datasetArgs, additionalDatasetArgs);
			}
			if (this.constructorParms.useJson === undefined) {
				this.constructorParms.useJson = false;
			}
			if (this.constructorParms.useJson === false) {
				if (this.constructorParms.getItems === undefined) {
					this.constructorParms.getItems = true;
				}
				if (this.constructorParms.getTasks === undefined) {
					this.constructorParms.getTasks = true;
				}
				if (this.constructorParms.retrieveItemProperties === undefined) {
					this.constructorParms.retrieveItemProperties = true;
				}
			} else {
				this.constructorParms.getTasks = true;
			}

			this._setupEventListening();
			if (this.constructorParms.getItems === true) {
				if (this.constructorParms.retrieveItemProperties === true) {
					this.sendCuriDataRequest(this.constructorParms.datasetId, djLang
							.hitch(this, this._datasetRetrieved), djLang.hitch(this, this._errorOnDatasetRetrieved),
							this.constructorParms.parameters, djLang.hitch(this, this._datasetItemsRetrieved), djLang
									.hitch(this, this._errorOnDatasetItemsRetrieved), true);
				} else {
					this.sendCuriDataRequest(this.constructorParms.datasetId, djLang
							.hitch(this, this._datasetRetrieved), djLang.hitch(this, this._errorOnDatasetRetrieved),
							this.constructorParms.parameters, djLang.hitch(this, this._datasetItemsRetrieved), djLang
									.hitch(this, this._errorOnDatasetItemsRetrieved));
				}
			} else {
				this.sendCuriDataRequest(this.constructorParms.datasetId, djLang.hitch(this, this._datasetRetrieved),
						djLang.hitch(this, this._errorOnDatasetRetrieved), this.constructorParms.parameters);
			}

		},
		/**
		 * return dataset object
		 */
		getDataset : function () {
			return this._dataset;
		},
		/**
		 * return tasks that belong to the dataset
		 */
		getTasks : function () {
			return this._tasks;
		},
		/**
		 * return all items that belong to the dataset
		 */
		getItems : function () {
			return this._items;
		},
		/**
		 * return a dataset(collection) attribute.
		 */
		getDatasetAttribute : function (key) {
			return this._dataset.getAttribute(key);
		},
		/**
		 * return an item by index (numeric)
		 */
		getItem : function (/*Integer*/index) {
			return this._items[index];
		},
		/**
		 * returns the value of a item property.
		 */
		getItemProperty : function (/*Dataset item*/item, /*Property id string*/property) {
			return item.getProperty(property).getValue();
		},
		/**
		 * goes through all items in data set and returns item that matches the
		 * value of a property. Use only when there is no duplicate values for
		 * the given property
		 */
		getItemFromProperty : function (propertyName, propertyValue) {
			var item = null;
			djArray.forEach(this.getItems(), function (entry, i) {
				var propValue;
				if (propertyName === "id") {
					propValue = entry._item.id;
				} else {
					propValue = entry.getProperty(propertyName).getValue();
				}
				if (propValue === propertyValue) {
					item = entry;
					return entry;
				}
			});
			return item;
		},
		/**
		 * Returns all properties for an item in JSON format
		 */
		getItemProperties : function (item) {
			var properties = {};
			var itemProps = item.getCachedPropertyIds(true);
			var i = 0;
			for (i = 0; i < itemProps.length; i++) {
				properties[itemProps[i]] = item.getProperty(itemProps[i]).getValue();
			}
			return properties;
		},
		/**
		 * return the dataset json data in JSON object format
		 */
		getDatasetJson : function () {
			return this._datasetJson;
		},
		updateServerJsonData : function (jsonObject, errorHandler, successHandler) {
			var jsonString = JSON.stringify(jsonObject);
			this.executeTask("executeJsonTask", {
				selectionParms : {
					jsonData : jsonString
				},
				onError : function (error, exception, ioArgs) {
					if (errorHandler) {
						errorHandler(arguments);
					}
				},
				onComplete : function (returnData) {
					if (successHandler) {
						if (returnData && returnData.result) {
							successHandler(JSON.parse(returnData.result));
						} else {
							successHandler();
						}
					}
				}
			});
		},

		/**
		 * Execute a dataset task. TODO: implement passing context Usage :
		 * executeTask(task_id OR task_object, { selectionParms : { YOUR_PROP_ID :
		 * YOUR_PROP_VALUE, YOUR_PROP_ID2 : YOUR_PROP_VALUE } });
		 */
		executeTask : function (task, args) {
			var taskArgs = {
				onComplete : function (result, kwArgs) {
				// Utils.Trace('task', 'success');
				},
				onError : function (error, exception, ioArgs) {
				//  Utils.Trace('task', 'error');
				},
				selectionParms : {}
			};
			djLang.mixin(taskArgs, args);
			if (typeof task === "string") {
				var i;
				for (i = 0; i < this._tasks.length; i++) {
					if (this._tasks[i].getId() === task) {
						task = this._tasks[i];
					}
				}
			}
			taskArgs.selection = new ibm.tivoli.navmodel.Selection({
				version : '1'
			});

			task.execute(taskArgs);
		},
		_setupEventListening : function () {
			this._eventCountToNotify = 1;
			this._currentEventCount = 0;
			if (this.constructorParms.getItems === true) {
				this._eventCountToNotify++;
			}
			if (this.constructorParms.getTasks === true) {
				this._eventCountToNotify++;
			}

		},
		_datasetRetrieved : function (dataset) {
			console.info('_datasetRetrieved');
			//  Utils.Trace(dataset);
			this._dataset = dataset;
			if (this.constructorParms.useJson === true) {
				this._datasetJson = JSON.parse(dataset.getAttribute("jsonData"));
			}
			if (this.constructorParms.getTasks) {
				var functions = {
					onComplete : djLang.hitch(this, this._datasetTasksRetrieved),
					onError : djLang.hitch(this, this._errorOnDatasetTasksRetrieved)
				};
				this._dataset.getTasks(functions);
			}
			if (this.constructorParms.getItems === true) {
				if (dataset._args.sort) {
					var store = new DatasetItemsStore({
						dataset : dataset,
						defaultProperties : dataset._args.defaultProperties
					});
					store.fetch({
						onComplete : djLang.hitch(this, this._datasetItemsRetrieved),
						onError : djLang.hitch(this, this._errorOnDatasetItemsRetrieved),
						sort : dataset._args.sort
					});
				}
			}
			this._eventReceived();
		},
		_errorOnDatasetRetrieved : function (error) {
			//TODO: allow callback on exploiters to get notified of specific errors
			this._retrievalError(error);
		},
		_datasetItemsRetrieved : function (items) {
			console.info('_datasetItemsRetrieved' + this.constructorParms.datasetId);
			this._items = items;
			this._eventReceived();
		},
		_errorOnDatasetItemsRetrieved : function (error) {
			//TODO: allow callback on exploiters to get notified of specific errors
			this._retrievalError(error);
		},
		_datasetTasksRetrieved : function (tasks) {
			console.info('_datasetTasksRetrieved');
			// Utils.Trace(tasks);

			this._tasks = tasks;
			this._eventReceived();
		},
		_errorOnDatasetTasksRetrieved : function (error, exception, ioArgs) {
			//TODO: allow callback on exploiters to get notified of specific errors
			this._retrievalError(error, exception, ioArgs);
		},
		_retrievalError : function (error, exception, ioArgs) {
			console.error('Dataset error for ' + this.datasetId);
			var msgHandler = new DefaultMessageHandler({});
			msgHandler.displayException(exception);
		},
		_eventReceived : function () {
			if (this.constructorParms.initReadyCallback == null) {
				return;
			}
			this._currentEventCount++;
			if (this._currentEventCount === this._eventCountToNotify) {
				if (this.constructorParms.useJson === true) {
					this.constructorParms.initReadyCallback(this._datasetJson);
				} else {
					this.constructorParms.initReadyCallback(this._items);
				}
			}
		},
		sendCuriDataRequest : function (navModelId, requestCallBack, requestErrorCallBack, parameters,
				nodesRetrievedCallBack, errorCallBack, retrieveItemProperties) {
			this._sendDataRequest(parameters, "/ibm/tivoli/rest", navModelId, requestCallBack, requestErrorCallBack,
					nodesRetrievedCallBack, errorCallBack, retrieveItemProperties);

		},
		_sendDataRequest : function (myParameters, myBaseUrl, myDataset, requestCallBack, requestErrorCallBack,
				nodesRetrievedCallBack, errorCallBack, retrieveItemProperties) {

			var args = {
				baseUrl : myBaseUrl,
				provider : "USMI",
				datasource : "",
				dataset : myDataset,
				parameters : myParameters,
				useDatastore : false,
				onComplete : requestCallBack,
				onError : requestErrorCallBack,
				defaultProperties : [ "all"
				]
			};
			djLang.mixin(args, this.datasetArgs);
			var dataset = new Dataset(args);
			if (requestCallBack) {
				dataset.onComplete = requestCallBack;
			}
			if (requestErrorCallBack) {
				dataset.onError = requestErrorCallBack;
			}
			if (nodesRetrievedCallBack) {
				var functions = {
					onComplete : nodesRetrievedCallBack
				};
				if (errorCallBack) {
					functions.onError = errorCallBack;
				}
				if (args.sort) {

				} else {
					dataset.getItemsAndLinks(functions);
				}
			}

		},
		destroy : function () {
			if (this._dataset) {
				this._dataset.destroy();
			}
			this._dataset = null;
			this._items = null;
			this._tasks = null;
		}

	});
});