/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/util/DefaultMessageHandler", [
	"dijit",
	"dojo",
	"dojox"
], function (dijit, dojo, dojox) {
	dojo.provide("ibm.usmi.util.DefaultMessageHandler");
	dojo.require("ibm.tivoli.navmodel.Exception");
	dojo.require("ibm.usmi.core.utils.Logger");
	dojo.require("ibm.usmi.widgets.FrameworkMessage");

	dojo.declare("ibm.usmi.util.DefaultMessageHandler", null, {

		// the logger
		_defaultMessageHandlerLogger : new ibm.usmi.core.utils.Logger("ibm.usmi.util", "DefaultMessageHandler"),
		_messages : null,
		constructor : function () {
			//Let's trace to the console window too
			if (this._defaultMessageHandlerLogger.isInfo()) {
				this._defaultMessageHandlerLogger.info("Creating new instance.", "constructor");
			}
			this._messages = [];
			this._messageResources = dojo.i18n.getLocalization("ibm.tivoli.tip.navwidget", "messages");
			registerMessageHandler(this); //main.js
		},

		displayException : function (/*Object*/ex) {

			var msgId = null;
			var msgSeverity = null;
			var msgText = null;
			var msgDescription = null;
			var moreDetails = null;
			var message = null;

			//Checking for existence of isInstanceOf before calling it because normal JS exception won't have an isInstanceOf
			//Also check that msgId, msgSeverity and msgText exist otherwise popup may not display
			//if (ex && ex.isInstanceOf && ex.isInstanceOf(ibm.tivoli.navmodel.Exception) && ex.getMsgId() && ex.getMsgSeverity() && ex.getMsgText()) {
			if (ex && ex.isInstanceOf && ex.isInstanceOf(ibm.tivoli.navmodel.Exception)) {				
				msgId = ex.getMsgId();
				msgSeverity = ex.getMsgSeverity();
				msgText = ex.getMsgText();
				msgDescription = ex.getMsgDescription();
				moreDetails = ex.getMoreDetails();
				
				
				//HACK - If CURI is returning a "user is not authorized message", make the message look better...
				if(msgId === "ATKRST105E") {
					var resources = dojo.i18n.getLocalization("ibm.usmi", "frameworkResources");
					msgId = null;
					msgText = resources.UserNotAuthorized;
					msgDescription = resources.UserNotAuthorizedDescription;					
				}

				//HACK - strip the msgID off the string since our REST classes are tacking it onto the beginning, which is ugly in the UI
				var str = msgText.substring(0, 3);
				if ((str === "ATK") || (str === "DNZ") || (str === "TCT")) {
					msgText = msgText.substring(msgText.indexOf(' ') + 1, msgText.length);
				}
			} else {

				//We have one of the following two messages to choose from. I don't think we're going to have enough in the exception to know if one of these is more appropriate than the
				//other. Also, I don't think we want to do the substitution for ${error_txt} since likely not translated.
				//
				//	- "TCTUI1002E":"An error has occurred.  The data provider could not return the data. ${error_txt}", // ${error_txt} will contain the error message returned from the server
				//	- "TCTUI1003E":"An error has occurred communicating with the server. ${error_txt}"  // ${error_txt} will contain the error message returned from the server
				msgId = "TCTUI1003E";
				msgSeverity = "Error";

				var messageTemplate = this._messageResources.TCTUI1003E;
				msgText = dojo.string.substitute(messageTemplate,
						{error_txt: ""}); //Doing an empty substitution since message in generic exception probably won't be translated
				//msgText = "An error has occurred communicating with the server.";
			}

			//Display the message
			var msgType = this._translateSeverity(msgSeverity);

			if (moreDetails) {
				message = this.displayMessageWithExternalLaunch(msgId, msgType, msgText, msgDescription, moreDetails, true);
			} else {
				message = this.displayMessage(msgId, msgType, msgText, msgDescription, true);
			}

			return message;
		},

		displayMessage : function (msgId, msgType, msgTitle, msgDescription, isCollapsed) {
			 var messageArea = dojo.byId('messageArea');

			 if (msgDescription == null) {
				 msgDescription = "";
			 }

			//var message = new idx.oneui.messaging.SingleMessage({
			var message = new ibm.usmi.widgets.FrameworkMessage({
				type : msgType,
				title : msgTitle,
				messageId : msgId,
				description : msgDescription,
				showViewDetails : false,
				collapsed : isCollapsed
			}).placeAt("messageArea", "last");
			this._messages.push(message);
			var container = dijit.byId("mainContentAreaLayout");
			container.resize();
			//Let's trace to the console window too
			if (this._defaultMessageHandlerLogger.isInfo()) {
				this._defaultMessageHandlerLogger.info(msgId + " " + msgType + " " + msgTitle, "displayTMSMessage");
			}

			return message;
		},

/*		displayTMSMessage : function (msgId, msgType, msgTitle, msgDescription, opResp) {
			var messageArea = dojo.byId('messageArea');
			if (msgDescription == null) {
				msgDescription = "";
			}
			//var message = new idx.oneui.messaging.SingleMessage({
			var message = new ibm.usmi.core.framework.FrameworkMessage({
				type : msgType,
				title : msgTitle,
				messageId : msgId,
				description : msgDescription,
				operatorResponse : opResp
			}).placeAt("messageArea", "last");
			this._messages.push(message);
			var container = dijit.byId("mainContentAreaLayout");
			container.resize();
			//Let's trace to the console window too
			if (this._defaultMessageHandlerLogger.isInfo()) {
				this._defaultMessageHandlerLogger.info(msgId + " " + msgType + " " + msgTitle, "displayTMSMessage");
			}

			return message;
		},*/
		displayTMSMessage : function (msgId, msgType, msgTitle, explanation, opResp) {
			var msgDescription = explanation + "<p>" + opResp + "</p>";
			return this.displayMessage(msgId, msgType, msgTitle, msgDescription, true);
		},

		displayMessageWithExternalLaunch : function (msgId, msgType, msgTitle, msgDescription, url) {
			return this.displayMessageWithExternalLaunch(msgId, msgType, msgTitle, msgDescription, url, null);
		},

		displayMessageWithExternalLaunch : function (msgId, msgType, msgTitle, msgDescription, url, launchLabel, isCollapsed) {
			var messageArea = dojo.byId('messageArea');

			if (msgDescription == null) {
				msgDescription = "";
			}

			//var message = new idx.oneui.messaging.SingleMessage({
			var message = new ibm.usmi.widgets.FrameworkMessage({
				type : msgType,
				title : msgTitle,
				messageId : msgId,
				description : msgDescription,
				moreDetailsURL : url,
				alternateMoreDetailsLabel : launchLabel,
				collapsed : isCollapsed
			}).placeAt("messageArea", "last");
			this._messages.push(message);

			var container = dijit.byId("mainContentAreaLayout");
			container.resize();
			//Let's trace to the console window too
			if (this._defaultMessageHandlerLogger.isInfo()) {
				this._defaultMessageHandlerLogger.info(msgId + " " + msgType + " " + msgTitle + " " + url,
						"displayMessageWithExternalLaunch");
			}

			return message;
		},

		displayMessageWithLaunchInContext : function (msgId, msgType, msgTitle, msgDescription, pageId, targets, url) {
			 return this.displayMessageWithLaunchInContext(msgId, msgType, msgTitle, msgDescription, pageId, targets, url, null, true);
		},

		displayMessageWithLaunchInContext : function (msgId, msgType, msgTitle, msgDescription, pageId, targets, url, launchLabel, isCollapsed) {
			var messageArea = dojo.byId('messageArea');

			if (msgDescription == null) {
				msgDescription = "";
			}

			//var message = new idx.oneui.messaging.SingleMessage({
			var message = new ibm.usmi.widgets.FrameworkMessage({
				type : msgType,
				title : msgTitle,
				messageId : msgId,
				description : msgDescription,
		        internalLaunch_pageId : pageId,
			    internalLaunch_targets : targets,
				internalLaunch_url : url,
				alternateMoreDetailsLabel : launchLabel,
				collapsed : isCollapsed
			}).placeAt("messageArea", "last");
			this._messages.push(message);

			var container = dijit.byId("mainContentAreaLayout");
			container.resize();
			//Let's trace to the console window too
			if (this._defaultMessageHandlerLogger.isInfo()) {
				this._defaultMessageHandlerLogger.info(msgId + " " + msgType + " " + msgTitle + " " + url,
						"displayMessageWithLaunchInContext");
			}

			return message;
		},


		/*displayTMSMessageInline : function (msgId, msgType, msgTitle, explanation, opResp) {
			var messageArea = dojo.byId('messageArea');

			if (msgDescription == null) {
				msgDescription = "";
			}

			var msgDescription = "<b>Explanation:</b><p>" + explanation + "</p><p>" + "<b>Operator Response</b><p>"
					+ opResp + "</p>";

			//var message = new idx.oneui.messaging.SingleMessage({
			var message = new ibm.usmi.core.framework.FrameworkMessage({
				type : msgType,
				title : msgTitle,
				messageId : msgId,
				description : msgDescription,
				showViewDetails : false
			}).placeAt("messageArea", "last");
			this._messages.push(message);

			var container = dijit.byId("mainContentAreaLayout");
			container.resize();
			//Let's trace to the console window too
			if (this._defaultMessageHandlerLogger.isInfo()) {
				this._defaultMessageHandlerLogger.info(msgId + " " + msgType + " " + msgTitle, "displayMessageInline");
			}

			return message;
		},*/

		clearAllMessages: function() {
			var i = 0;
			var count = this._messages.length;
			for (; i < count; ++i) {
				this._messages[i].destroyRecursive();
			}
			this._messages = [];
		},

		clearMessage : function (message) {
			var i;
			var count = this._messages.length;
			for (i = 0; i < count; ++i) {
				if (message === this._messages[i]) {
					this._messages[i].destroyRecursive();
					this._messages.splice(i, 1);
					break;
				}
			}
		},

		_translateSeverity : function (msgSeverity) {
			if (this._defaultMessageHandlerLogger.isFinest()) {
				this._defaultMessageHandlerLogger.fine("msgSeverity = " + msgSeverity, "_translateSeverity");
			}

			var type = "error";
			if (msgSeverity === "Error") {
				type = ibm.usmi.util.DefaultMessageHandler.ERROR;
			} else if (msgSeverity === "Warning") {
				type = ibm.usmi.util.DefaultMessageHandler.WARNING;
			} else if (msgSeverity === "Info") {
				type = ibm.usmi.util.DefaultMessageHandler.INFO;
			}
			return type;
		},

		destroy : function () {
			this.clearAllMessages();
			this.inherited(arguments);
		}
	});

	/*
	 * Message type. Can be 'error', 'warning', 'success', 'information', 'critical', 'attention', 'compliance'
	 */
	ibm.usmi.util.DefaultMessageHandler.ERROR = "error";
	ibm.usmi.util.DefaultMessageHandler.WARNING = "warning";
	ibm.usmi.util.DefaultMessageHandler.INFO = "information";
	ibm.usmi.util.DefaultMessageHandler.SUCCESS = "success";
	ibm.usmi.util.DefaultMessageHandler.CRITICAL = "critical";
	ibm.usmi.util.DefaultMessageHandler.ATTENTION = "attention";
	ibm.usmi.util.DefaultMessageHandler.COMPLIANCE = "compliance";

});
