/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define(
		"ibm/usmi/widgets/TreeSelector", [
			"dojo/_base/kernel",
			"dojo/_base/declare",
			"dojo/_base/lang",
			"dojo/_base/array",
			"dojo/topic",
			"dojo/_base/connect",
			"dijit/_WidgetBase",
			"dijit/layout/ContentPane",
			"dijit/layout/BorderContainer",
			// TWL imports
			"ibm/tivoli/navmodel/Dataset",
			"ibm/usmi/util/DefaultMessageHandler",
			"ibm/tivoli/tip/TCTTable",
			"ibm/tivoli/tip/navwidget/tree/dijit/TreeTableView",
			"ibm/tivoli/tip/navwidget/dijit/NavWidgetPluggableView",
			"ibm/tivoli/navmodel/Selection",
			// NextGen imports
			"ibm/usmi/util/BaseWidget",
			"ibm/usmi/widgets/util/Constants",
			"ibm/usmi/core/utils/_UsmiBrowserPersistence",
			"ibm/usmi/core/utils/Logger",
			"ibm/usmi/widgets/util/ResourceNavigatorDAHelper",
			"ibm/usmi/widgets/util/DefaultContextMenuProvider",
			"ibm/usmi/widgets/util/ResourceMenuProviderHelper"
		],
		function (dojo, declare, lang, array, topic, connect, _WidgetBase, ContentPane, BorderContainer, Dataset,
				DefaultMessageHandler, TCTTable, TreeTableView, NavWidgetPluggableView, Selection, BaseWidget,
				Constants, _UsmiBrowserPersistence, Logger, ResourceNavigatorDAHelper, DefaultContextMenuProvider,
				ResourceMenuProviderHelper) {

			return declare(
					[
						_WidgetBase,
						BaseWidget,
						BorderContainer,
						NavWidgetPluggableView,
						_UsmiBrowserPersistence
					],
					{

						_attempts : 0,
						treemodel : null,

						// attributes that can be set in constructor, html, or url tag
						style : "width:100%;height:100%;",

						// Defaults
						DEFAULT_COLLECTION : "RESOURCES_TREE_SELECTOR",

						///////////////////////////////////////////////////////////////////
						// private members
						///////////////////////////////////////////////////////////////////

						// the logger
						_treeSelectorLogger : null,

						// handle to TWL TreeTableView
						_tree : null,

						selectorTitle : null,

						selectorHome : null,

						// task selection widget path key passed in from dashbar selector
						defaultTaskSelectionPathKey : null,

						// task selection widget path passed in from dashbar selector
						defaultTaskSelectionPath : null,

						// task selection widget id passed in from dashbar selector
						defaultTaskSelectionWidget : null,

						_navRules : null,

						includeGroupNode : true,

						defaultActionHelper : null,

						// cmProvider: Custom context menu provider (must be provided on construction to be used)
						cmProvider : null,

						//Subscriptions to clean up
						_subscriptions : null,

						// Id of resource group currently selected
						_prevSourceToken : "",

						// default widget definition: found com.ibm.usmi.console.dojo.data\src\main\resources\widgetdefs\widgets
						_DEFAULT_WIDGET_DEF_INFO : {
							className : "ibm.tivoli.navmodel.TreeTableWidgetDef",
							baseUrl : Constants.BASE_REST_URL,
							provider : "any",
							datasource : "any",
							widget : "treetable-0.1.0",
							definition : "UsmiTreeTable-0.1.0"
						},

						constructor : function (/*constructor_kwArgs*/kwArgs) {
							// summary:
							//        Constructs Tree Selector object.

							//enabling logger for convenience. uncomment to use
							//dojoConfig.trace["ibm.usmi.widgets"]="finest";
							this._treeSelectorLogger = new Logger("ibm.usmi.widgets", "TreeSelector");

							if (this._treeSelectorLogger.isFinest()) {
								this._treeSelectorLogger.entering("", "constructor");
							}

							//        this.loadWidgetDefs();
							//We don't have a widget def specific to this view, so let's signal to
							//super class not to try and load any widget defs on our behalf
							this._usesWidgetDefinition = true;
							this.groupID = this.getInitArg(Constants.TREE_SELECTOR_GROUP);
							//only overwrite if defined, should make a baseWidget method for this
							if (this.getInitArg("INCLUDE_GROUP_NODE") !== undefined) {
								this.includeGroupNode = this.getInitArg("INCLUDE_GROUP_NODE");
							}

							if (this._treeSelectorLogger.isFinest()) {
								this._treeSelectorLogger.exiting("", "constructor");
							}
						},

						postCreate : function () {
							// summary:
							//         Called after a widget's dom has been setup
							// tags:
							//        protected
							//        this.initPersistence();

							this._subscriptions = [];

							this.inherited(arguments);

							var selectorTitleArg = this.getInitArg("selectorTitle"), selectorHomeArg = this
									.getInitArg("selectorHome"), defaultTaskSelectionWidgetArg = this
									.getInitArg("defaultTaskSelectionWidget");
							if (selectorTitleArg) {
								this.selectorTitle = selectorTitleArg;
							}

							if (selectorHomeArg) {
								this.selectorHome = selectorHomeArg;
							}

							if (defaultTaskSelectionWidgetArg) {
								this.defaultTaskSelectionWidget = defaultTaskSelectionWidgetArg;
							} else {
								this.defaultTaskSelectionWidget = Constants.DEFAULT_TASK_SELECTION_WIDGET;
							}

							// construct dataset, which once oncomplete, tree creation is kicked off
							this._makeDataset();

						},

						startup : function () {
							// summary:
							//         Called after any DOM fragments have been actually added to the document
							// tags:
							//        protected
							this.inherited(arguments);
						},

						destroy : function () {
							// summary:
							//        Clean up any objects the tree selector is hanging onto

							if (this._treeSelectorLogger.isFinest()) {
								this._treeSelectorLogger.entering("", "destroy");
							}
							var idx = 0;
							for (; idx < this._subscriptions.length; idx++) {
								this._subscriptions[idx].remove();
							}
							this._subscriptions.length = 0;

							this.inherited(arguments);

							if (this._treeSelectorLogger.isFinest()) {
								this._treeSelectorLogger.exiting("", "destroy");
							}
						},

						_parseTreeNavRules : function (dataset) {
							if (this._treeSelectorLogger.isFinest()) {
								this._treeSelectorLogger.entering("", "_parseTreeNavRules");
							}

							var navstring, navobject;
							navstring = dataset.getAttribute("TreeSelectorRules");
							if (navstring) {
								navobject = JSON.parse(navstring);
								this._navRules = navobject;
							}

							if (this._treeSelectorLogger.isFinest()) {
								this._treeSelectorLogger.exiting("", "_parseTreeNavRules");
							}

						},

						_makeTree : function (dataset) {
							// summary:  
							//        Construct TWL TreeTable view with passed in dataset, process sourcetoken for 
							//		pre-selection

							if (this._treeSelectorLogger.isFinest()) {
								this._treeSelectorLogger.entering("", "_makeTree");
							}

							//Destroy old tree
							if (this._tree) {
								this.removeChild(this._tree);
								this._tree.destroyRecursive();
								this._tree = null;
							}

							var stateString = null, state, hashSourceToken;

							if (this.widgetDefinition) {
								if (this.selectorTitle) {
									// Set the title of tree from the parameter passed in
									this.widgetDefinition.getDataSet()[0].columns[0].label.value = this.selectorTitle;
								} // If the dataset returns a title, set it to the tree.
								else if (dataset._dataset.label) {
									this.widgetDefinition.getDataSet()[0].columns[0].label.value = dataset._dataset.label;
								}
							}

							// create a default context menu provider
							this.defaultActionHelper = new ResourceNavigatorDAHelper({
								noOp : true
							});

							// get state information out of sessin persistence and set on tree
							// TODO once fix in in from TWL do state stuff
							//stateString = this.getData("treeKey", true);
							if (stateString) {
								state = dojo.fromJson(stateString);
								this._tree = new TreeTableView({
									id : "_treeTableView" + this.id,
									style : "width:100%;height:100%;",
									dataset : dataset,
									removeContextMenu : true,
									daHelper : this.defaultActionHelper,
									widgetDefinition : this.widgetDefinition,
									state : state
								});
							} else {
								this._tree = new TreeTableView({
									id : "_treeTableView" + this.id,
									style : "width:100%;height:100%;",
									dataset : dataset,
									removeContextMenu : true,
									daHelper : this.defaultActionHelper,
									widgetDefinition : this.widgetDefinition
								});
							}

							this.addChild(this._tree);

							// listen for when Tree is in charge, and unselecto tree topper selection
							this._subscriptions.push(topic.subscribe("TREETOPPERINCHARGE", dojo.hitch(this,
									this._removeSelection)));

							// Listen for hash updates
							this._subscriptions.push(topic.subscribe(Constants.TOPIC_HASH_CHANGE, dojo.hitch(this,
									this._hashChanged)));

							//Listen for selection change
							dojo.connect(this._tree.selection, Constants.SELECTION_CHANGE, dojo.hitch(this,
									this._treeSelectionChangedHandler));

							// get sourceToken from hash, and reopen / seelct branch if passed in
							hashSourceToken = this.getUrlHashParm("sourceToken");

							// Override _initTableBuilder so that we can connect to onTreeTableLoad event.
							// This is needed so we can know when the data is there, in order to auto-expand nodes.
							var old = this._tree._initTableBuilder;
							this._tree._initTableBuilder = lang.hitch(this,
									function (/*String*/providerId, /*String*/
									datasourceId, columnIds, datasetColumns, extraColData) {
										old.call(this._tree, providerId, datasourceId, columnIds, datasetColumns,
												extraColData);

										var timer = null;
										dojo.connect(this._tree._table, "onRefresh", lang.hitch(this, function () {
											// listen for table refreshes so we can detect if sourceToken is removed from tree
											// we're going to get a ton of these in a short time period so schedule a timer 
											// and if more come in, reset the timer
											if (timer) {
												clearTimeout(timer);
											}

											timer = setTimeout(lang.hitch(this, function () {
												timer = null;
												var grid = this._tree._grid;

												// make sure root node is expanded
												var row = grid.model.byIndex(0);
												var rItem = row.item;
												if (rItem) {
													grid.tree.expand(rItem.getId());
												}

												// if item is no longer in tree, move selection to root node
												var item = grid.model.byId(this._prevSourceToken);
												if (!item) {
													this._tree.selection.setSelectedObjects([ rItem
													], null, null, [ rItem.getId()
													]);
												}

											}), 800);
										}));

										// set the prevSourceToken so we dont kick off another by hashlistener
										this._prevSourceToken = hashSourceToken;
										// expand/preselect the tree if sourcetoken was specified on url 
										this._initialSelection(dataset, hashSourceToken);

									});

							this._parseTreeNavRules(dataset);

							if (this._treeSelectorLogger.isFinest()) {
								this._treeSelectorLogger.exiting("", "_makeTree");
							}
						},

						_makeDataset : function () {
							// summary:
							//        Get the dataset that was passed in on URL or constructor, default to RESOURCES_TREE_SELECTOR
							//        Construct TWL TreeTable view and set dataset

							if (this._treeSelectorLogger.isFinest()) {
								this._treeSelectorLogger.entering("", "_makeDataset");
							}

							var msgHandler, _processDataSet = function (dataset) {
								// now we have the dataset, lets create the tree
								this._makeTree(dataset);
							};

							if (!this.providerID) {
								// default
								this.providerID = Constants.DEFAULT_PROVIDER;
							}

							if (!this.selectorHome) {
								// default
								this.selectorHome = this.DEFAULT_COLLECTION;
							}

							if (!this.groupID) {
								// default
								this.groupID = "ROOT";
							}

							try {
								this.dataset = new Dataset({
									onComplete : dojo.hitch(this, _processDataSet),
									baseUrl : Constants.BASE_REST_URL,
									provider : this.providerID,
									datasource : "",
									dataset : this.selectorHome,
									parameters : {
										INCLUDE_GROUP_NODE : this.includeGroupNode
									},
									//parameters: {},
									defaultProperties : []
								});
							} catch (ex) {
								//There was a problem creating the dataset
								msgHandler = new DefaultMessageHandler({});
								msgHandler.displayException(ex);
							}

							if (this._treeSelectorLogger.isFinest()) {
								this._treeSelectorLogger.exiting("", "_makeDataset");
							}
						},

						_treeSelectionChangedHandler : function (objectContext) {
							//this selection handler gets called twice, once with 0 selection and 
							//once with real selection
							//this if guards the real function b/c settimeout always passes the latter parm
							//TODO: learn a better way to control what gets passed thru setTimeout             
							if (objectContext.getItems() && objectContext.getItems().length > 0) {
								// get of render thread so mouse click event can complete
								setTimeout(dojo.hitch(this, this._selectionChangedHandler, objectContext), 1);
							}
						},

						_selectionChangedHandler : function (objectContext) {
							// summary:
							//        Called when a node is selected in the the tree selector
							// tags:
							//        callback

							if (this._treeSelectorLogger.isFiner()) {
								this._treeSelectorLogger.entering("", "_treeSelectionChangedHandler");
							}

							var task, _processNavNewDataSet, _handleNavNewDataSetError;
							var sourceToken, myHashUpdateObject, nodeType, item, foundMatch, _processNavRule;
							var localHashObject = {}, retargetInfoObject = {}, datasetPublished = false;

							// with the communication between the topper and the tree, when topper tells the tree
							// to deselect, it fires of a selectionChange, so we're getting called sayin the tree is in
							// charge

							if (this._tree._table && this._tree._table.getSelectedRowCount() > 0) {
								// tell tree I'm in charge
								topic.publish("TREEINCHARGE", "true"); //tell tree topper I am in charge

								_processNavNewDataSet = function (dataset) {
									if (this._treeSelectorLogger.isFiner()) {
										this._treeSelectorLogger.finer("publish dataset: " + dataset.id,
												"_treeSelectionChangedHandler");
									}
									retargetInfoObject[Constants.RETARGET_INFO_DATASET] = dataset;
									topic.publish(Constants.TOPIC_RETARGET_INFO, retargetInfoObject); //tell workarea to switch it up
								};

								_handleNavNewDataSetError = function (error, request) {
									//There was a problem creating the dataset
									var msgHandler = new DefaultMessageHandler({});
									msgHandler.displayException(error);
								};

								foundMatch = false;
								item = objectContext.getItems()[0];
								nodeType = item.getAttribute("type");

								_processNavRule = function (navRule) {
									//could almost use for/in here, 
									//but the mapping to java to js isn't exact science
									retargetInfoObject[Constants.HASH_TASK_SELECTION] = navRule.taskWidgetId;

									if (navRule.taskParms) {
										if (lang.isObject(navRule.taskParms)) {
											//need to cycle through hashparms and add individually to the hash
											for ( var parmKey in navRule.taskParms) {
												retargetInfoObject[parmKey] = navRule.taskParms[parmKey];
											}
										} else {
											this._treeSelectorLogger
													.severe("TreeSelectorRules taskparms must be an object with key value pairs");
											//show error here too?
										}
									}
									if (navRule[Constants.HASH_TASK_PATH_KEY] && navRule[Constants.HASH_TASK_PATH]) {
										retargetInfoObject[Constants.HASH_TASK_PATH_KEY] = navRule[Constants.HASH_TASK_PATH_KEY];
										retargetInfoObject[Constants.HASH_TASK_PATH] = navRule[Constants.HASH_TASK_PATH];
									}
								};

								if (this._navRules && lang.isArrayLike(this._navRules)) {
									array.forEach(this._navRules, function (navRule) {
										if (!foundMatch && navRule.nodeType === nodeType) {
											foundMatch = true;
											_processNavRule(navRule);
											//enf not visible. see if i can pass it in
											//enf this._treeSelectorLogger.finest("***found nodetype in our map from array loop");
											//TODO: also need to break, could use array.some but may not be worth it
										}
									});
								} else if (this._navRules && this._navRules.nodeType === nodeType) {
									this._treeSelectorLogger.finest("***found nodetype in our map");
									foundMatch = true;
									_processNavRule(this._navRules);
								} else {
									this._treeSelectorLogger
											.finest("***nodetype NOT in map. Will check default nav hint");
								}

								if (!foundMatch) {
									var oid, theNode = objectContext.getItems()[0];
									if (theNode.getType() === "Group") {
										this._treeSelectorLogger.finest("new group check and publishing of oid");
										// lets publish the groupOID: string
										oid = theNode.getId();
										retargetInfoObject[Constants.HASH_TASK_PATH_KEY] = this.defaultTaskSelectionPathKey;
										retargetInfoObject[Constants.HASH_TASK_PATH] = this.defaultTaskSelectionPath;
										retargetInfoObject[Constants.HASH_TASK_SELECTION] = this.defaultTaskSelectionWidget;
										retargetInfoObject[Constants.COLLECTION] = "GROUP_OID:" + oid;
									} else {
										//get default task more efficiently
										task = objectContext.getItems()[0].getDefaultTask();
										//use task info to set the hash so workArea can do work
										if (task && task.isNavigation()) {
											//This should be handled by the generic navigate below,
											//but if the RELATIONSHIP isn't in the related resources list, it will fail. badly.
											//see UsmiNavigationHint.java for the differences in processing of baserules vs general rules 
											if (task.getId().startWith("RELATIONSHIP:")) {
												this._treeSelectorLogger.finest("***Found default nav hint. "
														+ task.getId());
												//not entirely sure this is needed var subbedString=task.getId().replace(/:/g, "_");
												retargetInfoObject[Constants.HASH_TASK_SELECTION] = "ibm/usmi/widgets/RelatedResourcesTable";
												retargetInfoObject[Constants.HASH_TASK_RELATIONSHIP] = task.getId();
											} else {
												this._treeSelectorLogger
														.finest("***Found default nav hint. navigate and publish dataset");
												retargetInfoObject[Constants.HASH_TASK_PATH_KEY] = this.defaultTaskSelectionPathKey;
												retargetInfoObject[Constants.HASH_TASK_PATH] = this.defaultTaskSelectionPath;
												retargetInfoObject[Constants.HASH_TASK_SELECTION] = this.defaultTaskSelectionWidget;
												datasetPublished = true;
												//navigate will give us back the new dataset. 
												//We'll in turn set that on the resource navigator
												objectContext.getDataset().navigate(
														{
															onComplete : dojo.hitch(this, _processNavNewDataSet),
															onError : dojo.hitch(this, _handleNavNewDataSetError),
															taskId : task.getId(),
															taskCategory : task.getCategory(),
															selection : this._buildClientNavModelSelection(
																	objectContext, objectContext.getDataset()
																			.getVersion())
														});
											}
										}
									}
								}
								// update sourceToken hash with selection
								sourceToken = objectContext.getItemIds()[0];
								this._prevSourceToken = sourceToken;
								localHashObject[Constants.SOURCE_TOKEN] = sourceToken;
								retargetInfoObject[Constants.SOURCE_TOKEN] = sourceToken;
								//append all our items to the hash, and get a string back that represents it
								myHashUpdateObject = this.updateHashObjectValues(localHashObject);

								// check to see if treeTopperId is set in hash, if so remove
								if (myHashUpdateObject.treeTopperId) {
									delete myHashUpdateObject.treeTopperId;
								}

								//actually set the hash
								this.setUrlHash(dojo.objectToQuery(myHashUpdateObject), true);

								//hackaround to handle the callbacks above that handled dataset publish to resnav
								if (!datasetPublished) {
									topic.publish(Constants.TOPIC_RETARGET_INFO, retargetInfoObject);
								}
								// store the current selection off into persistence so it can be retrieved later.
								// TODO once fix in in from TWL do state stuff
								//            stateObj = this._tree._table.getState();
								//            if(stateObj) {
								//                if (this._treeSelectorLogger.isFiner()) {
								//                    this._treeSelectorLogger.finer("save off stateObj", "_treeSelectionChangedHandler");
								//                }
								//                this.setData("treeKey", stateObj, true);
								//            }
							}
						},

						_hashChanged : function (hash) {
							var _sourceToken = this.getUrlHashParm(Constants.SOURCE_TOKEN);
							// should only get in this if statement in if the hash update was external 
							// i.e bookmark, external link, manual url change...
							if (_sourceToken && (this._prevSourceToken !== _sourceToken)) {
								// clear previous selection
								this._removeSelection();
								// store the _sourceToken
								this._prevSourceToken = _sourceToken;
								// change treeSelection
								this._initialSelection(this.dataset, _sourceToken);
							}
						},

						//***********************************************************
						// override NavWidgetPluggableView methods
						//***********************************************************
						_createWidgetDefinition : function () {
							// summary:
							//        Called by NavWidgetPluggableView, modify behavior
							// tags:
							//        private

							//Call super class which will load widget def
							this.inherited(arguments);

							//Look at widget def for attributes we need
							if (this.widgetDefinition) {
								this.widgetDefinition.getToolbar().hidden = true;
							}
						},

						_getDefaultWidgetDefInfo : function () {
							// summary:
							//        Load the default widget definition
							// tags:
							//        private
							return this._DEFAULT_WIDGET_DEF_INFO;
						},

						_onClickHandler : function (/*ibm.tivoli.navmodel.Item*/item) {
							// summary:
							//        Called when a node is selected in the the tree selector
							// tags:
							//        callback
							this._handlingSelectionChangedCallback = true;

							if (!this.selection.isChanging()) {
								this.selection.setSelectedItem(item);
								this._treeSelectionChangedHandler(item);
							}

							this._handlingSelectionChangedCallback = false;
						},

						//NOTE: _buildClientNavModelSelection is copied from AsynchContextMenu... could be a utility
						_buildClientNavModelSelection : function (objectContext, /*Float*/version) {
							// summary:
							//        Coming soon

							var currentSelection = null;
							if (objectContext) {
								currentSelection = new Selection({
									version : version
								});
								currentSelection.addItems(objectContext.getItemIds());
								currentSelection.addLinks(objectContext.getLinkIds());
								currentSelection.addRelationships(objectContext.getRelationshipIds());
							}
							return currentSelection;
						},

						//NOTE: _initialSelection
						_initialSelection : function (/*ibm.tivoli.navmodel.dataset*/dataset, /*String*/srcToken) {
							// summary:
							//        Coming soon
							dataset.getItems({
								onComplete : dojo.hitch(this, this.setItems, srcToken),
								onError : function (error, exception, ioArgs) {
									if (this._treeSelectorLogger.isWarn()) {
										this._treeSelectorLogger.warn("setting initial selection failed", "_initialSelection");
									}
								}
							});

						},

						setItems : function (/*String*/srcToken, /*ibm.tivoli.navmodel.Item[]*/items) {
							if (items) {
								// Return if grid is null
								if (!this._tree._grid) {
									if (this._treeSelectorLogger.isWarn()) {
										this._treeSelectorLogger.warn("Grid is null", "setItems");
									}
									return;
								}

								try {
									var grid = this._tree._grid, mytree = this._tree, row, rItem;

									if (srcToken) {
										// need to see if the src token is from this dataset
										var item = this._matchItem(items, srcToken);

										// when toggling between dashhbar items, the previos dashbarItems source
										// token is on the url, in that case lets default to the first item in tree
										if (item != null) {
											if (item.isOrigin()) {
												// since is origin, grid has it; get the row to expand and select it
												var rowIthink = grid.model.byId(item.getId());
												if (rowIthink) {
													rItem = rowIthink.item;
													mytree._grid.tree.expand(rItem.getId());

													// Tell the table to select the items
													mytree.selection.setSelectedObjects([ rItem
													], null, null, [ rItem.getId()
													]);

												}
											} else {
												// now we have a child selected, lets find out who his origin is
												// to do so we have to get each origin item get his children and see
												// if he is the parent of the source token passed in
												var originItem = null;
												for ( var j = 0; j < items.length; j++) {
													originItem = items[j];
													if (originItem.isOrigin()) {
														originItem.getRelatedItems({
															onComplete : function (relatedItems, kwArgs, originItem) {
																for ( var i = 0; i < relatedItems.length; i++) {
																	if (relatedItems[i].getId() === srcToken) {
																		// we found the parent, lets expand him
																		mytree._grid.tree.expand(originItem.itemId);
																		mytree.selection.setSelectedObjects(
																				[ relatedItems[i]
																				], null, null, [ relatedItems[i]
																						.getId()
																				]);
																		break;
																	}
																}
															}
														});
													}
												}
											}

										} else {
											// source token from url, was from previous dashbar; select / expan the first one
											row = this._tree._grid.model.byIndex(0);
											rItem = row.item;
											this._tree._grid.tree.expand(rItem.getId());

											// Tell the table to select the items
											this._tree.selection.setSelectedObjects([ rItem
											], null, null, [ rItem.getId()
											]);
										}

									} else {
										// no source token, select / expand the first one
										row = this._tree._grid.model.byIndex(0);
										rItem = row.item;
										this._tree._grid.tree.expand(rItem.getId());

										// only select if treeTopperId is NOT on url.   If treeTopperId is on url
										// then initial selection will be one of the treetoppers
										// Tell the table to select the items
										if (this.getUrlHashParm('treeTopperId') == null) {
											this._tree.selection.setSelectedObjects([ rItem
											], null, null, [ rItem.getId()
											]);
										}
									}

								} catch (err) {
									this._treeSelectorLogger.warn("***attempt to preselect tree node: " + err);
								}
							}
						},

						_matchItem : function (items, srcToken) {
							// first lets check the dataset to see if the sourcetoken matches an item id
							for ( var i = 0; i < items.length; i++) {
								if (items[i].getId() === srcToken) {
									return items[i];
								}
							}

							// Secondary check, if the item is lazyload && is an origin node, we need to get the related items
							// then check.. 
							var oItem = null;
							var foundItem = null;
							for ( var j = 0; j < items.length; j++) {
								oItem = items[j];
								if (oItem.isOrigin() && oItem.isLazyLoad()) {
									oItem.getRelatedItems({
										onComplete : function (relatedItems, kwArgs) {
											for ( var k = 0; k < relatedItems.length; k++) {
												if (relatedItems[k].getId() === srcToken) {
													// we found the node
													// todo we know the orgin so should we return it, to avoid another call?
													foundItem = relatedItems[k];
													break;
												}
											}
										},
										sync : true
									});
								}
							}

							return foundItem;
						},

						_removeSelection : function () {
							if (this._tree._table) {
								this._tree._grid.select.row.clear();
							}
						}
					});
		});