/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/widgets/WorkArea", [
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dojo/hash",
	"dojo/topic",
	"dijit/_WidgetBase",
	"dijit/layout/ContentPane",
	"dijit/layout/StackContainer",
	// TWL imports
	"ibm/tivoli/navmodel/Dataset",
	"ibm/usmi/util/DefaultMessageHandler",
	// NextGen imports
	"ibm/usmi/util/BaseWidget",
	"ibm/usmi/widgets/util/Constants",
	"ibm/usmi/core/utils/Logger"
], function (dojo, declare, hash, topic, _WidgetBase, ContentPane, StackContainer, Dataset, DefaultMessageHandler,
		BaseWidget, Constants, Logger) {

	return declare([
		_WidgetBase,
		BaseWidget,
		StackContainer
	],
			{

				// Defaults
				DEFAULT_COLLECTION : "RESOURCE_DETAILS_COLLECTION",

				///////////////////////////////////////////////////////////////////
				// private members
				///////////////////////////////////////////////////////////////////

				// array holding constructed widgets
				_widgets : null,

				// Id of widget currently shown in work area
				_currentTaskWidgetID : "",

				// relationship string, for resource table if shown in work area
				_isRelatedResource : false,

				// the logger
				_workAreaLogger : null,

				// kind of a hack, to see if selectorHome is set, use vs default
				selectorHome : null,

				// item gotten from the dataset, another pairing to detailsSelector
				item : null,

				//save topic handle so we can clean up
				topicHandle : null,

				// dataset from url
				dataset : null,

				constructor : function (/*constructor_kwArgs*/kwArgs) {
					//enabling logger for convenience. uncomment to use
					//dojoConfig.trace["ibm.usmi.widgets"] = "finest";
					this._workAreaLogger = new Logger("ibm.usmi.widgets", "WorkArea");
					if (this._workAreaLogger.isFinest()) {
						this._workAreaLogger.entering("", "constructor");
					}

					this._widgets = [];

					// check to see if selectorHome is specified, if so we want to use that
					this.selectorHome = this.getInitArg(Constants.SELECTOR_HOME);

					if (this._workAreaLogger.isFinest()) {
						this._workAreaLogger.exiting("", "constructor");
					}
				},

				startup : function () {
					//Call superclass
					this.inherited(arguments);

					//retarget info events
					this.topicHandle = topic
							.subscribe(Constants.TOPIC_RETARGET_INFO, dojo.hitch(this, this.retargeted));

					// let everyone we are open for business
					topic.publish(Constants.TOPIC_READY_WORKAREA, "");

				},

				postCreate : function () {
					//Call superclass
					this.inherited(arguments);

				},

				destroy : function () {
					// summary:
					//        Clean up any objects the workarea is hanging onto
					if (this._workAreaLogger.isFinest()) {
						this._workAreaLogger.entering("", "destroy");
					}
					if (this.topicHandle) {
						this.topicHandle.remove();
						this.topicHandle = null;
					}
					if (this._widgets) {
						var i = 0;
						for ( var key in this._widgets) {
							this._widgets[key].destroyRecursive();
							if (dijit.byId(key)) {
								dijit.byId(key).destroyRecursive();
							}

						}
						this._widgets.length = 0;
					}
					this.inherited(arguments);

					if (this._workAreaLogger.isFinest()) {
						this._workAreaLogger.entering("", "destroy");
					}
				},

				_makeWidget : function (/*String*/widgetID, /*object*/infoObj) {
					// summary:
					//	Called in response to a hash update.   
					//	Checks to see if widget already has been created
					//	and exists in _widgets array, if not creates, adds then displays it.
					var loadDone, widgetKey, currentWidget, pathKey, pathMod;
					if (this._workAreaLogger.isFinest()) {
						this._workAreaLogger.entering("", "_makeWidget");
					}

					widgetKey = this._getWidgetKey(widgetID, infoObj);
					currentWidget = this._widgets[widgetKey];
					//Do we have the widget already?
					if (!currentWidget) {
						if (this._workAreaLogger.isFiner()) {
							this._workAreaLogger.finer("Not the current widget, need to construct: " + widgetID,
									"_makeWidget");
						}

						//create widget function after requires are ready
						loadDone = function (currentWidget) {
							currentWidget = new currentWidget({
								id : widgetKey,
								style : Constants.CSS_MAX_SIZE,
								targetInfo : infoObj
							});
							if (!currentWidget) {
								currentWidget = new ContentPane({
									title : widgetID,
									content : "### PAGE CONTENT ###  " + widgetID
								});
							}

							// add widget to stack container and_widgets array
							this.addChild(currentWidget);
							this._widgets[widgetKey] = currentWidget;

							// set selected to current shown
							this.selectChild(currentWidget, true);
						};

						try {
							//check to see if taskPath was specified, 
							//if so need to add it to require statement
							// so can load module path
							if (infoObj[Constants.HASH_TASK_PATH] && infoObj[Constants.HASH_TASK_PATH_KEY]) {
								pathKey = infoObj[Constants.HASH_TASK_PATH_KEY];
								pathMod = infoObj[Constants.HASH_TASK_PATH];

								// add path to require
								dojo.registerModulePath(pathKey, pathMod);
								require([ widgetID
								], dojo.hitch(this, loadDone));
								//dojo isnt resolving the variable, its just passing it in
								//require({paths:{pathKey:
								//"/com.ibm.usmi.console.samples/js/usmi-console-samples"}}
								//,[widgetID], dojo.hitch(this, loadDone));
							} else {
								//require classes as needed on URL and create dijits
								require([ widgetID
								], dojo.hitch(this, loadDone));
							}
						} catch (err) {
							// log error
							if (this._workAreaLogger.isError()) {
								this._workAreaLogger.error("Unexpected error: " + err, "_makeWidget");
							}
						} //end catch
					} else {
						//We already had the widget.
						//set selected to current shown
						this.selectChild(currentWidget, true);
						//just show him. He'll figure the rest out based on hash, or the retarget coming elsewhere
						if (this._workAreaLogger.isFine()) {
							this._workAreaLogger.fine("already had the widget! showing him now: " + widgetID);
						}

					}

					if (this._workAreaLogger.isFinest()) {
						this._workAreaLogger.exiting("", "_makeWidget");
					}
				},

				_getWidgetKey : function (/*String*/widgetID, /*dojo.hash*/infoObj) {
					// summary:
					//        Create unique key based off widget ID, and hash parms

					var key = widgetID;
					// check to see if relationship was specified on the hash, if so use it to
					// construct unique id.   If it was not specified, just return the widget id as key
					if (infoObj[Constants.HASH_TASK_RELATIONSHIP]) {
						this._isRelatedResource = true;
						//	key = key + infoObj[Constants.HASH_TASK_RELATIONSHIP] + infoObj[Constants.SOURCE_TOKEN];
					}

					// set showing releated resource to false
					else {
						this._isRelatedResource = false;
						//key = key + infoObj["collectionID"] + infoObj[Constants.SOURCE_TOKEN];
					}

					//making the key just the widget ID. This determines if you're re-creating it, 
					///or just caching it in the stack. For things like related resourcs, 
					//just cache one copy and change the dataset
					return key;
				},

				//event contains info on what to create as child
				retargeted : function (retargetInfo) {
					this._currentTaskWidgetID = retargetInfo[Constants.HASH_TASK_SELECTION];
					this._makeWidget(retargetInfo[Constants.HASH_TASK_SELECTION], retargetInfo);
				}

			});
});
