/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
// Finder TaskDataStore
define("ibm/usmi/widgets/finder/FinderContextSearch", [
	"dojo/_base/lang",
	"dojo/_base/declare",
	"dojo/topic",
	"ibm/usmi/util/DataStore",
	"./TaskDataStore",
	"dojo/i18n!ibm/usmi/nls/frameworkResources",
	"ibm/tivoli/tip/navwidget/dojo/NavWidgetPluggableViewObjectContext"
], function (lang, declare, topic, DataStore, TaskDataStore, MRI, Selection) {

	return declare([], {
		//Fields for context finder (searches for targetted tasks against current selection in resource view / chassis map)
		//datastore points to all systems dataset, with the current selection
		contextFinderDatastore : null,
		//current system selection that the context finder will search against
		selectedOids : null,
		//flag to show progress while we search for targetted tasks against current selection
		retrievingTasks : false,
		needToFetchTasks : true,
		_contextTempItems : null,
		finderObj : null,
		constructor : function (finderObj) {
			this.finderObj = finderObj;
			this.contextFinderDatastore = new DataStore({
				datasetId : 'GROUP:ALL_ENDPOINTS',
				getTasks : false,
				initReadyCallback : lang.hitch(this, this.initializeContextDataset)
			}, {
				defaultProperties : [ "DisplayName"
				]
			});
		},
		initializeContextDataset : function () {
			this.contextFinderDatastore.getDataset().addEventListener(this,
					lang.hitch(this, this.refreshContextDataset));
		},

		refreshContextDataset : function () {
		//TODO: if all resources group changes, context finder will need to update the datastore to include those changes
		},

		//Called when chassis map / nav widget of resources change selections. Context finder gets initialized with new context
		changeSelection : function (oids) {
			var validResources = "", listOfResources, i = 0;
			listOfResources = oids.split(",");
			for (; i < listOfResources.length; i++) {
				if (this.contextFinderDatastore.getItemFromProperty("id", listOfResources[i])) {
					if (validResources !== "") {
						validResources += ",";
					}
					validResources += listOfResources[i];
				}
			}
			if (this.selectedOids === validResources) {
				return;
			}
			this.finderObj.closeSearchDetailDlg();
			this.selectedOids = validResources;
			this.needToFetchTasks = true;
		},

		//context finder tasks have been retrieved for current selection, populate the finder cache for this
		datasetTasksRetrieved : function (tasks, args) {
			if (this._contextTempItems) {
				this._contextTempItems = null;
			}
			this._contextTempItems = [];
			this.createRecursiveContextTaskList(tasks, '', true, 'General');
			this.finderObj._cachedDataStores.contexttasks = new TaskDataStore('contexttasks', this._contextTempItems);
			this.retrievingTasks = false;
		},

		//iterate recursively through all context tasks and their child tasks and build the display info and launch info into the cache
		createRecursiveContextTaskList : function (tasks, taskList, first, category) {
			var i, icon;
			for (i = 0; i < tasks.length; i++) {
				var task = tasks[i];
				if (task.getType() === 'separator' || task.getType() === "navigation") {
					continue;
				} else if (task.getType() === 'cascade') {
					this.createRecursiveContextTaskList(task.getChildTasks(), '', first, task.getLabel());
					continue;
				}
				var taskId = task.getId();
				if (taskId === null || taskId.length === 0) {
					if (i === 0) {
						first = true;
					}
					continue;
				}

				if (task.getLaunchJavascript().indexOf("launchType:'ISCPage'") !== -1) {
					icon = "js/ibm/usmi/core/framework/themes/images/ac10_opennewtab_24.png";
				} else {
					icon = "js/ibm/usmi/core/framework/themes/images/ge10_bullet_24.png";
				}

				this._contextTempItems.push({
					id : taskId,
					label : task.getLabel(),
					category : category,
					icon : icon
				});
				task.parentCategory = category;
				this.finderObj._indexedItems[taskId + "#_context"] = task;
			}
			return;
		},

		datasetRetrievedError : function () {

		},
		executeTask : function (task) {
			var jsonLaunchParms = task.getLaunchJavascript();
			var launchParms = dojo.fromJson(jsonLaunchParms);
			var objectContext = {
				selection : new Selection({
					dataset : this.contextFinderDatastore.getDataset(),
					itemIds : [ this.selectedOids
					]
				}),
				dataset : this.contextFinderDatastore.getDataset()
			};
			launchParms.selection = objectContext.selection;
			task.actionID = task.getId();
			if (launchParms.topic) {
				topic.publish(launchParms.topic, launchParms, task, objectContext);
			}
		},
		determineContextSearch : function () {
			if (this.selectedOids && this.selectedOids !== "") {
				if ((this.retrievingTasks === false && !this.finderObj._cachedDataStores.contexttasks)
						|| this.needToFetchTasks === true) {
					if (this.finderObj._cachedDataStores.contexttasks) {
						this.finderObj._cachedDataStores.contexttasks.destroy();
						this.finderObj._cachedDataStores.contexttasks = null;
					}

					this.retrievingTasks = true;
					this.needToFetchTasks = false;

					this.contextFinderDatastore.getDataset()._fetchTasks({
						onComplete : lang.hitch(this, this.datasetTasksRetrieved),
						onError : lang.hitch(this, this.datasetRetrievedError),
						itemIds : this.selectedOids
					});
				}
			}
		},
		destroy : function () {
			if (this.contextFinderDatastore) {
				this.contextFinderDatastore.getDataset().removeEventListener(this);
				this.contextFinderDatastore.destroy();
				this.contextFinderDatastore = null;
			}
			this._contextTempItems = null;
		}
	});
});
