/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/widgets/table/DefaultTreeTableView", [
	"dojo/_base/declare",
	"dojo/_base/kernel",	
	"dojo/on",
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/aspect",	
	"ibm/tivoli/tip/navwidget/tree/dijit/TreeTableView",
	"ibm/usmi/widgets/table/ConfigureColumnsDialog",
	"ibm/usmi/core/utils/Logger",
	"ibm/usmi/widgets/table/DefaultTablePersistenceMixin"
], function (declare, dojo, on, lang, array, aspect, TreeTableView, ConfigureColumnsDialog, 
		Logger, DefaultTablePersistenceMixin) {

	return declare([ TreeTableView //, DefaultTablePersistenceMixin
	], {

		// the logger
		_defaultTreeTableViewLogger : new Logger("ibm.usmi.widgets.table", "DefaultTreeTableView"),
		
		//@188637 START	uncomment to enable tree table persistence		
//		augmentGridParams: function (gridParams) {
//			gridParams = this.inherited(arguments);
//			gridParams = this.persistenceGridParams(this._dataStore, gridParams);
//			return gridParams;
//		},
		//@188637 START	

		// AbstractTableView._initTableBilder connects to onTableLoad, and inadvertently shows the filter bar.
		// This aspect is called afterward, and hides the filter bar again.
		_initTableBuilder : function () {
			this.inherited(arguments);		

			//@188637 START	uncomment to enable tree table persistence			
//			if (this._table) {
//				this._table.setFilter(null, true);
//				var searchField = null;
//				if (this._table._toolbar && this._table._toolbar.quickFilter) {
//					searchField = this._table._toolbar.quickFilter;
//				}
//				this.initPersistence(this, this._grid, searchField);
//			}
			//@188637 END
			
			var changeStyle = function (row) {
				if (!this.layout._isCollapsable) {
					this.inherited(arguments);
					return;
				}
				row.customClasses = (row.selected ? " SelectorTreedojoxGridRowSelected" : "")
						+ (row.over ? " SelectorTreedojoxGridRowOver" : "");
				this.focus.styleRow(row);
				this.edit.styleRow(row);
			};
			on(this._table._grid, "styleRow", changeStyle);

			//Hack in our column chooser
			if (this._table && this._table._toolbar) {
				//console.log("setting the custom Column Chooser");
				this._table._toolbar._configureColumn = function () {
					this.configureTableDialog = new ConfigureColumnsDialog({
						grid : this.treeTable._grid
					});
					this.configureTableDialog.show();
				};
			}
		},
				
		handleLinkClickAction : function (viewId, objectId, propId, objectType) {
			// summary:
			//		Method to override base implementation so we get notified and can
			//      handle when a link is clicked in a table cell
			if (this._defaultTreeTableViewLogger.isFinest()) {
				this._defaultTreeTableViewLogger.entering("", "handleLinkClickAction");
				this._defaultTreeTableViewLogger.finest("handling link click for table view - prop: " + propId,
						"handleLinkClickAction");
			}
			var object, property;

			//ask the dataset for the correct object based on the objects id.
			//since we have already rendered these items, the value should be
			//cached and should be quick.
			switch (objectType) {
			case "ibm.tivoli.navmodel.Item":
				object = this.dataset.getItem({
					itemId : objectId
				});
				break;
			case "ibm.tivoli.navmodel.Link":
				object = this.dataset.getLink({
					itemId : objectId
				});
				break;
			case "ibm.tivoli.navmodel.Relationship":
				object = this.dataset.getRelationship({
					itemId : objectId
				});
				break;
			default:
				if (this._defaultTreeTableViewLogger.isFinest()) {
					this._defaultTreeTableViewLogger.finest("unknown objectType: " + objectType,
							"handleLinkClickAction");
				}
			}

			if (object) {
				property = object.getProperty(propId);
			}

			if (object && property) {
				this.daHelper.executeDefaultAction(this, property);
			}

			if (this._defaultTreeTableViewLogger.isFinest()) {
				this._defaultTreeTableViewLogger.exiting("", "handleLinkClickAction");
			}
		}
	});
});
