/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

define("ibm/usmi/widgets/util/PropertyRenderer", ["dojo/_base/kernel",
        "dojo/_base/declare",
        "dijit/ProgressBar",
        //TWL imports
        "ibm/tivoli/navmodel/Property",
        "ibm/tivoli/navmodel/Status",
        "ibm/tivoli/navmodel/ItemStyle",
        "ibm/tivoli/navmodel/Item",
        "ibm/tivoli/navmodel/Link",
        "ibm/tivoli/navmodel/Relationship",
        "ibm/tivoli/tip/navwidget/dojo/NavWidgetUtil",
        "ibm/tivoli/tip/navwidget/properties/PropertyRenderer",
        // Next Gen
        "ibm/usmi/widgets/util/Constants"],

    function (dojo, declare, ProgressBar,
            Property, Status, ItemStyle, Item, Link, Relationship, NavWidgetUtil, PropertyRenderer,
            Constants) {

    return declare([PropertyRenderer], {

        //***************************
        // "Public Methods"
        //***************************

        constructor: function() {
            // summary:
            //        Coming soon
            // description:
            //        Coming soon
            //Call superclass
            this.inherited(arguments);
        },

        getRenderedPropertyNode: function(/*ibm.tivoli.navmodel.Item*/ object, prop, plugviewref, rowID, cell) {
            //first check if there is a valid value
            switch(prop.getValueState()){
            case "ok":

                    switch (prop.getValueType()) {
                    case "itemName":

                        var contentNode = dojo.doc.createElement("div"),
                            threshold, altText;

                        //a threshold callback is defined, so use it to check if we need
                        //to threshold this value and ADD an image which we normally wouldnt render
                        if(this.propThresholdCallback){
                            threshold = this.propThresholdCallback(prop, object);

                            //only if an image is defined on the threshold for this value
                            if(threshold && threshold.icon){
                                altText = (threshold.label ? threshold.label.value : null);
                                if (!altText) {
                                    altText = prop.getDisplayValue();
                                }

                                //render the node, and add it
                                dojo.place(this._getImageContentNode( plugviewref, prop, object, "", "16px", "16px"), contentNode, "last", altText);
                            }
                        }

                        if(prop.getValueUnits()) {
                            dojo.place(this._getLabelContentNode(prop.getDisplayValue() + " " + prop.getValueUnits(), prop, object, plugviewref ), contentNode,"last");
                        } else {
                            dojo.place(this._getLabelContentNode(prop.getDisplayValue(), prop, object, plugviewref  ), contentNode,"last");
                        }

                        //color background if thresholding needed
                        this._thresholdBackground(cell, prop, object);

                        return contentNode;

                        default:
                    //probably need to add some error handling in here in case the input value is invalid
                    }
                    break;
            default:

            }

            return this.inherited(arguments);
        },

        getResourceImage: function(plugviewref, prop, object, imagePath, height, width, altText){
            //create the image node
            var fullImageNode,
                linkNode,
                imageNode= dojo.doc.createElement("img"),
                threshold,
                objType;

            //a threshold callback is defined,
            if(this.propThresholdCallback){
            //use it to check if we need to see if we need to threshold this image
                threshold = this.propThresholdCallback(prop, object);
                if(threshold && threshold.icon){
                    //threshold condition met, so use the threshold image instead
                    //dojo.attr(imageNode, "src", plugviewref._cleanUpImagePath(threshold.icon));
                    imagePath = plugviewref._cleanUpImagePath(threshold.icon);
                }
            }


            //add the attributes to the img node based on the object style
            dojo.attr(imageNode, "src", plugviewref._cleanUpImagePath(imagePath));
            dojo.attr(imageNode, "height", height);
            dojo.attr(imageNode, "width", width);
            dojo.attr(imageNode, "style", "vertical-align:sub");
            dojo.attr(imageNode, "hspace", "3px");
            dojo.addClass(imageNode, "tctGridCellImg"); //253021
            if (altText) {
                dojo.attr(imageNode, "alt", altText);
            }

            if(prop.getTask()){
                linkNode = dojo.doc.createElement("a");

                dojo.attr(linkNode, Constants.ON_CLICK, "ibm.tivoli.tip.navwidget.properties.handleLinkClick(this);return false;" );
                dojo.attr(linkNode, "id", object.getId() + "_"+ prop.getId()+".img.link");
                dojo.attr(linkNode, "propid", prop.getId());
                dojo.attr(linkNode, "objectid", object.getId());
                dojo.attr(linkNode, "viewid", plugviewref.id);
                //check to see which type of object this is and set on ojbecttype attr for link
                objType = null;
                if(object instanceof Item){
                    objType = "ibm.tivoli.navmodel.Item";
                }
                else if(object instanceof Link){
                    objType = "ibm.tivoli.navmodel.Link";
                }
                else if(object instanceof Relationship){
                    objType = "ibm.tivoli.navmodel.Relationship";
                }
                dojo.attr(linkNode, "objecttype", objType);


                dojo.attr(linkNode, "href", "#");

                dojo.place(imageNode, linkNode, "last");

                fullImageNode = linkNode;

            }else{
                fullImageNode = imageNode;
            }


            return fullImageNode;

        }
    });
});
