// wrapped by build app
define("ibm/usmi/widgets/util/Utils", ["dijit","dojo","dojox","dojo/require!ibm/usmi/widgets/util/Constants"], function(dijit,dojo,dojox){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
dojo.provide("ibm.usmi.widgets.util.Utils");
//dojo.require("util.Format");
dojo.require("ibm.usmi.widgets.util.Constants");

/** copied from TPC Util.js **/
ibm.usmi.widgets.util.Utils = {
    /* Transforms an object with multiple attributes (arrays), into an array with
     * attributes with those names.
     *
     * Input: { id: [0,1,2], dontcare: "value", names: ['a','b','c'] }
     * Output: [{ id: 0, names: 'a'}, { id: 1, names: 'b' }, { id: 2, names: 'c' }]
     */
    Object2Array: function _Object2Array(obj) {
        var fields = [], out = [], sz, field, attr, i, j;

        // Gather names of fields that are arrays
        for (attr in obj) {
            if (dojo.isArray(obj[attr])) {
                fields.push(attr);
            }
        }
        if (fields.length) {
            // get final length of output array by measuring first field's size
            sz = obj[fields[0]].length;
            for (i = 0; i < sz; i += 1) {
                out[i] = {};
                for (j in fields) {
                    field = fields[j];
                    out[i][field] = obj[field][i];
                }
            }
        }
        return out;
    },

    /*
     * Current time since epoch in milliseconds
     */
     currentTimeMillis: function() {
        return (new Date()).getTime();
     },

    /* Convert an error object returned by the xhr errback to a TPC Message object
     * Returned message object format
     * {result : {
     *      type:
     *      msgId:
     *      time:
     *      text:
     * }}
     */
    toBSMMessage: function(error /*object*/) {
        if (!error || !error.message) {
            return error;
        } else {
            var bsmMsg = {
                result : {
                    type: "E",
                       msgId: "000001E",  //TODO: Get the correct ID for this message
                    time: "timestampe",//util.Format.formatTimestamp(new Date()),
                    text: error.message
                }
            };
            return bsmMsg;
        }
    },



    /**
     * The following three functions are testing the status strings based off the following convention:
     *     S:Success
     *   I:Information
     *   W:Warning
     *   E:Error
     *
     * These functions are useful when checking the result of a task.
     * General format for the result:
     *    result{ status: statusString, msgId: msgIdString, text: msgTextString }
     */
    /* Checks the string to see if its a success string*/
    isSuccess: function _isSuccess(/*String*/status){
        return status === "S";
    },

    isInformation: function _isInformation(/*String*/status){
        return status === "I";
    },

    /* Checks the string to see if its a warning string*/
    isWarning: function _isWarning(/*String*/status){
        return status === "W";
    },

    /* Checks the string to see if its an error string*/
    isError: function _isError(/*String*/status){
        return status === "E";
    },

    getLaunchURL: function _getLaunchURL(/*String*/ page) {
        return ibm.usmi.widgets.util.Constants.LAYOUT_PAGE + page;
    },

    /* Launch a page */
    launchPage: function _launchPage(/*String*/ page){
        window.location = ibm.usmi.widgets.util.Utils.getLaunchURL(page);
    },

    /* Launch a page in a seaparate browser window/tab */
    launchNewPage: function _launchNewPage(/*String*/ page){
        window.open(ibm.usmi.widgets.util.Utils.getLaunchURL(page));
    },

    /**
     * Abbreviate the string.
     * Usage: tpc.util.Utils.abbreviate("HelloThere", {min: 5, max: 6, center: false});
     * @param st - A string
     * @param optional - {min: aNumber, max: aNumber, center: boolean}
     */
    abbreviate: function _abbreviate(/*String*/ st, /*Object*/ optional) {
        if (st) {
            optional = dojo.mixin({min: 5, max: 23, center: true}, optional);
            var len = st.length,
            max = Math.max(optional.min, optional.max),
            sansEllipsis, n1, n2,suffix;
            if (len > max) {
                sansEllipsis = (max - 3);

                if (optional.format && optional.format === (" (x)") ) {
                    n2 =  st.lastIndexOf("(");
                    if (n2 > 0) {
                        suffix = "... " + st.substring(n2);
                        n1 = max - suffix.length;
                        return st.substring(0, n1) + suffix;
                    } else {
                        return st.substring(0, sansEllipsis) + "...";
                    }
                } else if (optional.center === true) {
                    n1 = Math.ceil(sansEllipsis/2);
                    n2 = len - (sansEllipsis - n1);
                    return st.substring(0, n1) + "..." + st.substring(n2);
                } else {
                    return st.substring(0, sansEllipsis) + "...";
                }
            }
        }
        return st;
    },

    /*
     * Compare val1 and val2 and, if they are equal or if both are null, undefined, empty or NaN,
     * then returns true, else returns false.
     */
    areValuesEqual: function (/*String|Number*/ val1, /*String|Number*/ val2, /*Boolean*/ trim) {
        if (trim) {
            val1 = this.trim(val1);
            val2 = this.trim(val2);
        }
        if (val1 === val2) {
            return true;    //Values are equal
        }
        //Values are not equal, but we must verify whether both are null, undefined, empty or NaN
        if ((val1 === null) || (typeof val1 === "undefined") || (val1 === "") ||
                (typeof val1 === "number" && isNaN(val1))) {
            //val1 is null, undefined, empty or NaN. Verify whether val2 is too.
            return (val2 === null) || (typeof val2 === "undefined") || (val2 === "") ||
                (typeof val2 === "number" && isNaN(val2));
        }
        return false;
    },

    /**
     * Returns the trimmed string, if the argument is a string, or the original argument otherwise
     */
    trim: function _trim(src) {
        return (typeof src === "string") ? dojo.trim(src) : src;
    },

    /**
     * Convert a plain text into HTML by replacing the characters having special meaning in HTML
     * by the corresponding escape sequence.
     */
    convertTextToHTML: function _convertTextToHTML(/*String*/ src) {
        return (typeof src === "string") ?
            src.replace(/&/gm, "&amp;").replace(/</gm, "&lt;").replace(/>/gm, "&gt;").replace(/"/gm, "&quot;").replace(/\n/g, "<br>")
            : src;
    },

    /**
     * Return the status code inside the given request object if it exists.  Return -1 if no status is found.
     */
    getRequestStatus: function(request) {
        return (request.xhr && request.xhr.status) ? request.xhr.status : -1;
    },

    /*
     * utility function to add and remove a class for a node
     */
    addAndRemoveClass : function(/* DOMnode*/ node, /*String */ classToAdd, /*String */ classToRemove) {

         dojo.removeClass(node, classToRemove);
         dojo.addClass(node, classToAdd);
    },


    contains: function(str1, str2, caseSensitive) {
        if(!caseSensitive) {
            str1 = str1.toLowerCase();
            str2 = str2.toLowerCase();
        }
        return str2.indexOf(str1) !== -1;
    }
};

});
